Object subclass: #GameEngine
	instanceVariableNames: 'gameMessages location locationList person doRun tagName process simProcess '
	classVariableNames: 'Instance '
	poolDictionaries: ''
	category: 'SimTech-Engine'!
!GameEngine commentStamp: 'JMC 10/22/2004 18:31' prior: 0!
This is the Game Engine.

Basically, the GameEngine is a simple Kernel in the form of a singleton Object.

As for how this works, the GameEngine basically gets messages of two types:
kernel message - message of great importance, usually user input
game message - message of lesser importance, usually simulation related or a
                   result of some kernel message.

The primary job of the Game Engine is to ensure that the flow of the simulation
is handled, as well as handle interaction between "modules" of the game.  In
addition, it must ensure that kernel messages (such as the user clicking wildly)
do not adversely affect the simulation, so it must avoid starvation of either
message queue.!


!GameEngine methodsFor: 'access' stamp: 'VU 11/18/2004 01:55'!
isRunning
	"Returns true if the game engine is running, and false if it is not running."
	^(process isNil)
		ifTrue: [ false. ]
		ifFalse: [ process isTerminated not. ].! !


!GameEngine methodsFor: 'communication' stamp: 'VU 10/30/2004 23:57'!
currentLocation
	"Returns the current Location"

	^ location.! !

!GameEngine methodsFor: 'communication' stamp: 'JMC 10/29/2004 22:35'!
currentPerson
	"Returns the currently selected person"

	^person.
! !

!GameEngine methodsFor: 'communication' stamp: 'dqn 11/1/2004 00:13'!
locationList
	^ locationList.! !


!GameEngine methodsFor: 'initialization' stamp: 'JMC 11/14/2004 20:53'!
buildUp
	"This is called from to instantiate any necessary variables"

	gameMessages := MessageQueue new.
	
	locationList _ Bag new.

	tagName _ 'GameEngine'.

	doRun := nil.! !

!GameEngine methodsFor: 'initialization' stamp: 'JMC 11/18/2004 19:51'!
deinit
	"Reinitialize the instance"

	gameMessages := nil.! !

!GameEngine methodsFor: 'initialization' stamp: 'JMC 11/18/2004 19:51'!
reinit
	"Reinitialize the instance"

	gameMessages := MessageQueue new.! !


!GameEngine methodsFor: 'message handling' stamp: 'VU 11/18/2004 01:48'!
fireCriticalMessage: message
	"Process the message. NOW!!"
	message process.! !

!GameEngine methodsFor: 'message handling' stamp: 'JMC 11/14/2004 16:17'!
fireGameMessage: message
	"Add a Message to the Game Message queue"
	
	gameMessages enqueue: message.
! !

!GameEngine methodsFor: 'message handling' stamp: 'JMC 11/18/2004 19:39'!
fireKernelMessage: message
	"Add a Message to the Kernel Message 'Queue'"
	"Since Kernel messages should be instantly executed, there's no need for a queue, it just instantly gets handled"

	"^self fireGameMessage: message.	"

	message process.! !

!GameEngine methodsFor: 'message handling' stamp: 'JMC 10/29/2004 20:14'!
fireSimulationMessage
	"Adds a simulation message to the gameMessage queue.
		This is wrapper function that chain calls the fireGameMessage method
		with the appropriate message for updating the simulation as the parameter."

	 "TODO: Create a new Simulation message and fire it"! !


!GameEngine methodsFor: 'modifiers' stamp: 'dqn 10/31/2004 22:48'!
addLocation: aLocation
	locationList add: aLocation.! !

!GameEngine methodsFor: 'modifiers' stamp: 'JMC 10/29/2004 22:43'!
changeLocation: newLocation
	"This will change the currentLocation as the GameEngine sees it"

	^self currentLocation: newLocation.! !

!GameEngine methodsFor: 'modifiers' stamp: 'JMC 10/29/2004 22:42'!
changePerson: newPerson
	"Change the current person"

	^self currentPerson: newPerson.
! !

!GameEngine methodsFor: 'modifiers' stamp: 'JMC 10/29/2004 22:42'!
currentLocation: newLocation
	"This will change the currentLocation as the GameEngine sees it"

	^location := newLocation.! !

!GameEngine methodsFor: 'modifiers' stamp: 'VU 10/30/2004 22:31'!
currentPerson: newPerson
	"Change the current person"

	person := newPerson.
	UIManager instance getSelectedItemMorph currentObject: newPerson.
! !

!GameEngine methodsFor: 'modifiers' stamp: 'JMC 10/30/2004 16:30'!
doRun: anObject
	"Set the doRun variable to anObject"

	doRun := anObject.
! !


!GameEngine methodsFor: 'scheduler' stamp: 'JMC 11/14/2004 15:55'!
handleNextMessage
	"This takes the necessary actions to handle the next Message"

	| next |
	
	next := self nextMessage.

	next isNil ifFalse: [ ^next process ] ifTrue: [ ^nil ].

	
	

! !

!GameEngine methodsFor: 'scheduler' stamp: 'JMC 11/18/2004 19:56'!
kill
	"Kill the process"

	process isNil ifFalse: [process terminate.].

	process := nil.

	simProcess isNil ifFalse: [simProcess terminate.].

	simProcess := nil.
	
	gameMessages := MessageQueue new.! !

!GameEngine methodsFor: 'scheduler' stamp: 'JMC 11/14/2004 16:20'!
nextMessage
	"Returns the next message to be handled as well as properly removes it from
		the ready queues".

	^gameMessages dequeue.
! !

!GameEngine methodsFor: 'scheduler' stamp: 'stuff 12/2/2004 03:17'!
resume
	"Resume the Game Engine process"

	process := [
					"[doRun isNil] whileFalse: ["
					[nil isNil] whileTrue: [
					
						"Transcript show: 'The Game Engine is running the loop';cr."
			
						self handleNextMessage isNil ifTrue: [ 
							"Transcript show: 'Message is nil, waiting';cr."
							(Delay forMilliseconds: 500) wait. 
						].
					]. 
				] newProcess.

	simProcess := [
					[nil isNil] whileTrue: [
						(Delay forMilliseconds: ((600 to: 1500) atRandom)) wait.
						GameEngine instance currentLocation worldObjectBag do: [:curr |
							(curr model isKindOf: Person) ifTrue: [
								((curr model currentAction isKindOf: DoNothing) or: [curr model currentAction isNil not or: [curr model finalMessage isNil not]]) ifTrue:
								[
									curr model invokeSimAction.
								].
								(curr model currentAction isKindOf: DoNothing) ifTrue:
								[
									curr model doCurrentAction.
								].
							].
						].
					].
				] newProcess.

	process resume.
	simProcess resume.! !


!GameEngine methodsFor: 'xml' stamp: 'dqn 10/31/2004 22:55'!
tagName
	^ tagName.! !

!GameEngine methodsFor: 'xml' stamp: 'VU 11/1/2004 12:24'!
writeXML: xmlWriter
	xmlWriter startTag: self tagName.	
	xmlWriter attribute: 'currentLocationID' value: location locationID asString.
	xmlWriter endTag.
	xmlWriter startTag: 'locations'; endTag.
	locationList do:
	[
		: element |
		element writeXML: xmlWriter.
	].
	xmlWriter endTag: 'locations'. 
	xmlWriter endTag: self tagName.
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

GameEngine class
	instanceVariableNames: ''!

!GameEngine class methodsFor: 'Singletonness' stamp: 'JMC 10/29/2004 18:40'!
delete
	"This will delete the current instance of the Singleton, this is primarily for debugging"

	^(Instance := nil).

	! !

!GameEngine class methodsFor: 'Singletonness' stamp: 'JMC 11/18/2004 19:48'!
instance
	"Return the current instance of the Singleton"

	Instance isNil ifTrue: [ Instance := super new. Instance buildUp.].

	^Instance.
! !

!GameEngine class methodsFor: 'Singletonness' stamp: 'JMC 10/29/2004 18:41'!
new
	"This should never be called due to the Singleton design pattern"

	^self error: 'Singletons cannot be instantiated, use initialize instead'.
! !


!GameEngine class methodsFor: 'initialization' stamp: 'JMC 11/14/2004 20:48'!
start
	"Start the GameEngine"

	GameEngine instance doRun: 'Look, Im running'.

	GameEngine instance resume.
! !

!GameEngine class methodsFor: 'initialization' stamp: 'JMC 11/14/2004 20:59'!
stop
	"Stop the Game Engine"

	Transcript show: 'The Game Engine is being stopped'; cr.

	GameEngine instance kill.
! !


!GameEngine class methodsFor: 'as yet unclassified' stamp: 'dqn 11/4/2004 19:10'!
instance: anInstance
	Instance _ anInstance.! !
