CriticalMessage subclass: #CreateBuildingMessage
	instanceVariableNames: 'coordinates originalLocation width height locationName doorPosition alignment floorType addBuilding '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-Message'!

!CreateBuildingMessage methodsFor: 'access' stamp: 'VU 11/27/2004 14:19'!
coordinates: aPoint
	coordinates := aPoint.! !

!CreateBuildingMessage methodsFor: 'access' stamp: 'VU 11/27/2004 14:19'!
location: aLocation
	originalLocation := aLocation.! !

!CreateBuildingMessage methodsFor: 'access' stamp: 'VU 11/27/2004 13:45'!
name
  ^ 'Create Location Here'.! !


!CreateBuildingMessage methodsFor: 'process' stamp: 'VU 12/2/2004 01:33'!
process
	"Processes the message"

	| widthStr heightStr posStr ftStr bldgStr |

	" *** Get location name *** "
	locationName := FillInTheBlank request: 'Please name the location:'
				initialAnswer: 'Location'.
	(locationName = '') ifTrue: [ "Cancel was pressed"
		^self.
	].

	" *** Get location width *** "
	widthStr := FillInTheBlank request: 'What will be the width of the location?'
				initialAnswer: '10'.
	(widthStr = '') ifTrue: [ "Cancel was pressed"
		^self.
	].
	width := widthStr asInteger.

	" *** Get location height *** "
	heightStr := FillInTheBlank request: 'What will be the height of the location?'
				initialAnswer: '10'.
	(heightStr = '') ifTrue: [ "Cancel was pressed"
		^self.
	].
	height := heightStr asInteger.

	" *** Get the floor type *** "
	ftStr := RadioButtonDialog
				request: 'What will be the flooring in this room?'
				withValues: { 'Grass'. 'Asphalt'. 'Carpet'. 'Tile'. 'Linoleum'. 'Parquet' }.
	(ftStr = '') ifTrue: [ "Cancel was pressed"
		^self.
	].
	floorType := self privateTypeFromString: ftStr.

	" *** Is it a building? *** "
	bldgStr := RadioButtonDialog
				request: 'Is it a building or a room?'
				withValues: { 'Building'. 'Room' }.
	(bldgStr = '') ifTrue: [ "Cancel was pressed"
		^self.
	].
	(bldgStr = 'Building') ifTrue: [
		addBuilding := true.
	] ifFalse: [
		addBuilding := false.
	].

	" *** Get position of the door back *** "
	posStr := RadioButtonDialog
				request: 'Where will the door back be?'
				withValues: { 'North-West'. 'South-West'. 'South-East'. 'North-East'. 'Center' }.
	(posStr = '') ifTrue: [ "Cancel was pressed"
		^self.
	].
	doorPosition := self privateDoorPositionFrom: posStr in: width@height.

	" *** Create room *** "
	self privateCreateRoom.! !


!CreateBuildingMessage methodsFor: 'private' stamp: 'VU 12/2/2004 02:53'!
privateCreateRoom
	"Creates the room"

	| room doorFrom building |

	"Create the new location:"
	room := Location new.
	room name: locationName.
	room width: width height: height.
	room floorType: floorType.

	"Create the building if we want to."
	addBuilding ifTrue: [
		building := Building new.
		building location: room.
		originalLocation addWorldObject: building at: coordinates.
	] ifFalse: [
		building := Portal new.
		building location: room.
		building alignment: (alignment + 2) \\ 5.
		(building alignment = 0) ifTrue: [
			building alignment: 1.
		].
	].

	"Create a door from the original location to the new location:"
	originalLocation addWorldObject: building at: coordinates.

	"Create a door from the new location back to the original location:"
	doorFrom := Portal new.
	doorFrom location: originalLocation.
	doorFrom alignment: alignment.
	room addWorldObject: doorFrom at: doorPosition.! !

!CreateBuildingMessage methodsFor: 'private' stamp: 'VU 12/1/2004 21:30'!
privateDoorPositionFrom: aString in: aDimension
	"Returns a point that contains the position of the door depending on where the user wants to put it and the dimensions of the room"

	| halfWidth halfHeight |

	width := aDimension x.
	height := aDimension y.
	halfWidth := width // 2.
	halfHeight := height // 2.

	(aString = 'North-West') ifTrue: [
		alignment := 3.
		^(1@halfHeight).
	].

	(aString = 'South-West') ifTrue: [
		alignment := 4.
		^(halfWidth@height).
	].

	(aString = 'South-East') ifTrue: [
		alignment := 1.
		^(width@halfHeight).
	].

	(aString = 'North-East') ifTrue: [
		alignment := 2.
		^(halfWidth@1).
	].

	alignment := 1.

	^(halfWidth@halfHeight).! !

!CreateBuildingMessage methodsFor: 'private' stamp: 'VU 12/1/2004 22:11'!
privateTypeFromString: aString
	"Creates a type floor index from string."
	"The following types are available: "
	"{ 'Grass'. 'Asphalt'. 'Carpet'. 'Tile'. 'Linoleum'. 'Parquet' }."

	(aString = 'Grass') ifTrue: [ ^1. ].
	(aString = 'Asphalt') ifTrue: [ ^2. ].
	(aString = 'Carpet') ifTrue: [ ^3. ].
	(aString = 'Tile') ifTrue: [ ^4. ].
	(aString = 'Linoleum') ifTrue: [ ^5. ].
	(aString = 'Parquet') ifTrue: [ ^6. ].

! !
