Object subclass: #Location
	instanceVariableNames: 'worldObjectBag locationArray locationID name floorType '
	classVariableNames: 'LocationID NextCoordinate '
	poolDictionaries: ''
	category: 'SimTech-Model'!
!Location commentStamp: 'JMC 10/29/2004 20:28' prior: 0!
Location contains the information the model needs to represent the current location.

This includes the area information (movable floor areas and other information for valid locations) as well
as the information regarding objects located within the area.  Note that the original composition 
included a LocationGrid composed of MapObjects, however that has been extracted as purely for the 
view, thus changing the structure of the Location.
!


!Location methodsFor: 'accessors' stamp: 'VU 12/2/2004 02:42'!
doorCount
	"Returns the number of doors in this location."

	| count |

	count := 0.

	worldObjectBag do: [ :wo |
		(wo model isKindOf: Portal) ifTrue: [
			count := count + 1.
		].
	].

	^ count.
! !

!Location methodsFor: 'accessors' stamp: 'JMC 11/17/2004 20:12'!
findEmptySpotAround: aPoint
  "Returns the coordinates of an empty spot somewhere close to the coordinates passed in."

  ^self findEmptySpotAround: (aPoint x) and: (aPoint y).! !

!Location methodsFor: 'accessors' stamp: 'VU 11/1/2004 16:25'!
findEmptySpotAround: x and: y
  "Returns the coordinates of an empty spot somewhere close to the coordinates passed in."

  | point center |

  center := x@y.

  (1 to: 4) do: [ :distance |
    NextCoordinate do: [ :coord |
      point := center + (distance * coord x)@(distance * coord y).
      (self inBounds: point) ifTrue: [
        (self getWorldObjectAt: point) ifNil: [ ^point. ].
      ].
    ].
  ].

  ^ nil.! !

!Location methodsFor: 'accessors' stamp: 'VU 12/1/2004 23:46'!
findObject: aWorldObject
	"Returns the map object associated with the world object"

	worldObjectBag do: [ :mo |
		(mo model = aWorldObject) ifTrue: [ ^mo. ].
	].

	^ nil.! !

!Location methodsFor: 'accessors' stamp: 'VU 11/1/2004 16:53'!
findPortalTo: aLocation
  "Returns a portal to the location passed in, or nil if no such portal exists."

  | model |

  worldObjectBag do: [ :wo |
    model := wo model.
    (model isKindOf: Portal) ifTrue: [
      (model location locationID = aLocation locationID) ifTrue: [ ^model. ].
    ].
  ].

  ^ nil.! !

!Location methodsFor: 'accessors' stamp: 'VU 12/1/2004 20:44'!
floorType
	"Returns the type of the floor used in this location."
	^ floorType.! !

!Location methodsFor: 'accessors' stamp: 'VU 12/1/2004 20:45'!
floorType: newFloorType
	"Sets the floor type used for this location to the new value passed in."
	floorType := newFloorType.! !

!Location methodsFor: 'accessors' stamp: 'dqn 10/30/2004 23:38'!
getWorldObjectAt: aPoint
	^locationArray at: (aPoint x) at: (aPoint y).! !

!Location methodsFor: 'accessors' stamp: 'VU 10/30/2004 18:48'!
height
  "Responds with the height of the map."

  ^ locationArray height.! !

!Location methodsFor: 'accessors' stamp: 'VU 11/1/2004 16:25'!
inBounds: aPoint
  "Checks whether the point passed in is within bounds of the map."

  | x y |

  x := aPoint x.
  y := aPoint y.

  (x < 1) ifTrue: [ ^false. ].
  (y < 1) ifTrue: [ ^false. ].
  (x > self width) ifTrue: [ ^false. ].
  (y > self height) ifTrue: [ ^false. ].

  ^ true.! !

!Location methodsFor: 'accessors' stamp: 'VU 11/6/2004 15:18'!
isCurrentLocation
  "Returns true if this location is actually the current location or false if it is not."

  | cl |

  cl := GameEngine instance currentLocation.

  ^ (self = cl).! !

!Location methodsFor: 'accessors' stamp: 'JMC 10/30/2004 21:28'!
isWalkable: aPoint
	"Check whether a position is walkable"

	^self isWalkable: (aPoint x) at: (aPoint y).! !

!Location methodsFor: 'accessors' stamp: 'JMC 12/1/2004 22:26'!
isWalkable: x at: y
	"Check whether a position is walkable"

	(((locationArray at: x at: y) isNil) or: [( ( (locationArray at: x at: y) model ) isKindOf: Movable)]) ifTrue: [
		^true.
	].
	
	^false. 
! !

!Location methodsFor: 'accessors' stamp: 'dqn 11/11/2004 02:47'!
locationArray
	^ locationArray! !

!Location methodsFor: 'accessors' stamp: 'dqn 10/31/2004 00:12'!
locationID
	^locationID.! !

!Location methodsFor: 'accessors' stamp: 'dqn 10/31/2004 23:48'!
locationID: id
	locationID _ id.! !

!Location methodsFor: 'accessors' stamp: 'VU 10/30/2004 19:00'!
locationObjects
  "Returns the bag of objects in the location"
  ^ worldObjectBag.! !

!Location methodsFor: 'accessors' stamp: 'VU 11/1/2004 15:24'!
name
  "Returns the name of the location"
  ^ name! !

!Location methodsFor: 'accessors' stamp: 'VU 11/1/2004 15:24'!
name: aString
  " Sets the name of the location to the new string "
  name := aString.! !

!Location methodsFor: 'accessors' stamp: 'dqn 10/31/2004 18:05'!
tagName
	^ 'location'.! !

!Location methodsFor: 'accessors' stamp: 'VU 10/30/2004 18:48'!
width
  "Responds with the width of the map."

  ^ locationArray width.! !

!Location methodsFor: 'accessors' stamp: 'dqn 11/1/2004 00:14'!
worldObjectBag
	^ worldObjectBag.! !


!Location methodsFor: 'initialization' stamp: 'VU 12/1/2004 22:12'!
initialize
	"Initialize the Location"
	
	worldObjectBag := Bag new.
	locationArray := nil.

	locationID _ LocationID + 1.

	name := 'Location'.
	floorType := 1.

	GameEngine instance addLocation: self.! !


!Location methodsFor: 'modifiers' stamp: 'JMC 10/30/2004 21:04'!
addWorldObject: worldObject
	"Add a World Object to the bag of World Objects"

	^self error: 'FUCK YOU ADDWORLDOBJECT'.
	
	! !

!Location methodsFor: 'modifiers' stamp: 'VU 12/1/2004 23:56'!
addWorldObject: worldObject at: aPoint
	"Add a World Object to the bag of World Objects"
	self addWorldObject: worldObject at: (aPoint x) at: (aPoint y).! !

!Location methodsFor: 'modifiers' stamp: 'VU 11/6/2004 15:19'!
addWorldObject: worldObject at: x at: y
	"Add a World Object to the bag of World Objects"

    | mapObject |

    mapObject := AnimatedMapObject onFrameset: (FramesetManager instance framesetFor: worldObject).
    mapObject stepTime: 300.
    mapObject coordinates: worldObject coordinates.
    mapObject model: worldObject.
	
	worldObjectBag add: mapObject.

	mapObject coordinates: (x@y).
	mapObject model coordinates: (x@y).

	locationArray at: x at: y put: mapObject.

	(worldObject isKindOf: Person) ifTrue: [
		worldObject changeLocation: self.
	].

	(self isCurrentLocation) ifTrue: [
		UIManager instance getLocationGrid addMorphFront: mapObject.
	].

	! !

!Location methodsFor: 'modifiers' stamp: 'VU 11/1/2004 16:32'!
enter: aPerson from: aLocation
  "Processes the entrance of a person from a location"

  | portal point |

  portal := self findPortalTo: aLocation.
  portal isNil
    ifTrue: [ point := self findEmptySpotAround: (self width // 2) and: (self height //2) ]
    ifFalse: [ point := self findEmptySpotAround: (portal coordinates x) and: (portal coordinates y) ].

  point ifNotNil: [ self addWorldObject: aPerson at: point x at: point y. ].! !

!Location methodsFor: 'modifiers' stamp: 'VU 11/17/2004 11:56'!
moveMapObject: toMove to: aPoint
	"Move an object to a new point"

	| x y oldCoord mapObject |

	worldObjectBag do: [ :value |
		(value model = toMove) ifTrue: [
			mapObject := value.
		].
	].	

	x _ aPoint x.
	y _ aPoint y.
	
	oldCoord _ mapObject coordinates.

	mapObject slideTo: aPoint.
	mapObject model coordinates: aPoint.

	locationArray at: x at: y put: mapObject.	

	locationArray at: oldCoord x at: oldCoord y put: nil.! !

!Location methodsFor: 'modifiers' stamp: 'JMC 10/29/2004 20:42'!
removePortal: deadPortal
	"Remove a portal from the current Location"

	^portalBag remove: deadPortal.
! !

!Location methodsFor: 'modifiers' stamp: 'JMC 10/30/2004 21:31'!
removeWorldObject: worldObject
	"Remove a worldObject from the current Location"

	worldObjectBag do: [ :value |
		(value model = worldObject) ifTrue: [
			| aPoint |
			
			worldObjectBag remove: value.

			aPoint := value coordinates.

			locationArray at: (aPoint x) at: (aPoint y) put: nil.			

			value delete.
			^ self.
		].
	].
! !

!Location methodsFor: 'modifiers' stamp: 'JMC 11/17/2004 20:00'!
setWalkable: x at: y isWalkable: canWalk
	"Change the walkable status of the position in the location"
	"Nil means is walkable"

	"^locationArray at: x at: y put: canWalk."

	^self error: 'Should not be called'.! !

!Location methodsFor: 'modifiers' stamp: 'JMC 11/17/2004 20:01'!
setWalkable: aPoint isWalkable: canWalk
	"Change the walkable status of the position in the location"
	"Nil means is walkable"

	"^locationArray at: (aPoint x) at: (aPoint y) put: canWalk."

	^self error: 'Dont call me'.! !

!Location methodsFor: 'modifiers' stamp: 'JMC 10/29/2004 22:54'!
width: newWidth height: newHeight
	"Sets the width and height of the locationArray"

	^locationArray _ Array2D width: newWidth height: newHeight.
 ! !


!Location methodsFor: 'xml' stamp: 'VU 11/1/2004 15:29'!
writeXML: xmlWriter
	xmlWriter startTag: self tagName. "open tag location"
     xmlWriter attribute: 'id' value: (locationID asString). 
	xmlWriter attribute: 'name' value: self name.
	xmlWriter endTag. "Close the tag with a >"
	
	xmlWriter startTag: 'locationArray'.
     xmlWriter attribute: 'width' value: (locationArray width asString). 
     xmlWriter attribute: 'height' value: (locationArray height asString). 
	xmlWriter endTag.	
	xmlWriter endTag: 'locationArray'.
	
	xmlWriter startTag: 'worldObjectBag'; endTag.
	worldObjectBag do:
	[
		: element |
		element model writeXML: xmlWriter.
	].
	xmlWriter endTag: 'worldObjectBag'.


	xmlWriter endTag: self tagName.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Location class
	instanceVariableNames: ''!

!Location class methodsFor: 'as yet unclassified' stamp: 'VU 11/1/2004 16:41'!
initialize
  "Initializes static variables"

  LocationID _ 0.
  NextCoordinate _ OrderedCollection new.
  NextCoordinate add: -1@-1.
  NextCoordinate add: 0@-1.
  NextCoordinate add: 1@-1.
  NextCoordinate add: 1@0.
  NextCoordinate add: 1@1.
  NextCoordinate add: 0@1.
  NextCoordinate add: -1@1.
  NextCoordinate add: -1@0.! !

!Location class methodsFor: 'as yet unclassified' stamp: 'VU 10/30/2004 19:14'!
new
	"Instantiate a new class"

	^super new initialize.


	! !


Location initialize!
