WorldObject subclass: #Person
	instanceVariableNames: 'name gender factors location items personalEffects currentAction destinationCoordinates path last pathTry finalMessage '
	classVariableNames: 'NextCoordinate '
	poolDictionaries: ''
	category: 'SimTech-Model'!
!Person commentStamp: 'ACK 11/5/2004 02:12' prior: 0!
The Person class defines People.  Specifically, this is their effects, effects given to them by items, the items they hold, and various other factors.

The Person class handles the application of effects to them from items as well as the effects given to others.!


!Person methodsFor: 'accessors' stamp: 'VU 11/6/2004 15:28'!
currentAction
  "Returns what the person is currently doing."
  ^currentAction.! !

!Person methodsFor: 'accessors' stamp: 'ACK 10/22/2004 18:56'!
factor: fact
	"Get the value of the game factor at fact.  Returns Error if non-existant"

	"Check the existance of the Key and error if non-existant"
	(factors includesKey: fact) ifFalse:
		[^'Error: Factor Does not Exist'.].

	^factors at: fact.! !

!Person methodsFor: 'accessors' stamp: 'dqn 11/22/2004 17:27'!
finalMessage
	^finalMessage! !

!Person methodsFor: 'accessors' stamp: 'dqn 11/22/2004 17:28'!
finalMessage: aMessage
	finalMessage _ aMessage.! !

!Person methodsFor: 'accessors' stamp: 'ACK 10/22/2004 18:56'!
gender
	"Get the gender of the person"

	^gender.! !

!Person methodsFor: 'accessors' stamp: 'ACK 10/22/2004 19:21'!
location
	"This will return the Person's current room"

	^location.! !

!Person methodsFor: 'accessors' stamp: 'ACK 10/22/2004 18:55'!
name
	"Get the name of the Person"

	^name.! !


!Person methodsFor: 'actions' stamp: 'ACK 10/29/2004 20:06'!
doAction: inAction
	"applies the effects of the action to the persons factors"

	(inAction effects) do: [:effect |
		self addPersonalEffect: effect].

! !

!Person methodsFor: 'actions' stamp: 'stuff 12/2/2004 03:19'!
doCurrentAction
	(currentAction isNil) ifTrue:
	[
		^ self.
	].

	((currentAction isKindOf: DoNothing) or: [self nextTo: (currentAction object coordinates)]) ifFalse:
	[
		|message|
		"current action is a real action"
		"if not next to an object, order to move first"
		Transcript show: 'doCurrentAction: current action is ', currentAction printString; cr.
		destinationCoordinates _ nil.
			self runTo: (currentAction object coordinates).
"		message _ RunPersonMessage new.
		message location: location.
		message coordinates: currentAction object coordinates.
		message owner: self.
		message time: 1.

		GameEngine instance fireGameMessage: message."
		^self.
	].
	currentAction doAction: self.
	self setAction: DoNothing new.
	! !

!Person methodsFor: 'actions' stamp: 'ACK 11/18/2004 00:59'!
interactionsFor: aWorldObject
	"returning a bag of actions that can be performed "

	| availableActions action |

	availableActions := Bag new.

	" add talk action if the object is a person "
	(aWorldObject isKindOf: Person) ifTrue: [
		| it |
		(self == aWorldObject) ifFalse: [
			it := InteractTalk new.

			it otherPerson: self.

			availableActions add: it.
		].
	].

    " if interacting with self, add the ability to drop items"
    (aWorldObject = self) ifTrue: [
		items do: [ :item |
			action := DropItemMessage new.
			action owner: aWorldObject.
			action time: 0.
			action item: item.
			availableActions add: action.
		].
	].

	^availableActions.! !

!Person methodsFor: 'actions' stamp: 'VU 12/2/2004 04:05'!
invokeSimAction
	"Cause the person to do something based on his/her factors/attributes/surroundings...etc."

	| point xcoor ycoor message |

	(currentAction isKindOf: DoNothing) ifFalse: [^self].

	"Randomly decide to do nothing"
	(((1 to: 3) atRandom) = 1) ifTrue: [^self].

	"Check to see if the person is really hungry"
	((self factor: 'hungry') = 'Error: Factor Does not Exist') ifFalse: [
		((self factor: 'hungry') > 100) ifTrue: [
			"pickup food if it is around"
			((self location) worldObjectBag) do: [:i |
				((i model) isKindOf: Food) ifTrue: [
					| drinkAction |
					drinkAction := EatAction new.
					drinkAction food: i model.
					self setAction: drinkAction.

"
					|pickupMessage|
					pickupMessage _ PickupItemMessage new.
					pickupMessage owner: self.
					pickupMessage item: (i model).
					pickupMessage time: 1.
					GameEngine instance fireGameMessage: pickupMessage.
"
					^self.
				].
			].
		].
	].

	"Check to see if the person is really sleepy"
	 ((self factor: 'sleep') = 'Error: Factor Does not Exist') ifFalse: [
		((self factor: 'sleep') > 100) ifTrue: [
			"Pickup coffee if it is around"
			((self location) worldObjectBag) do: [:i |
				((i model) isKindOf: Coffee) ifTrue: [
					| drinkAction |
					drinkAction := DrinkAction new.
					drinkAction coffee: i model.
					self setAction: drinkAction.

"
					| pickupMessage |
					pickupMessage _ DrinkAction new.
					pickupMessage owner: self.
					pickupMessage item: (i model).
					pickupMessage time: 1.
					GameEngine instance fireGameMessage: pickupMessage.
"
					^self.
				].	
			].
		].
	].

	"Start to move but only make it one step b/c of fatigue."
"	(location inBounds: (path at: last)) ifFalse:
		[
			^ self.
		]."

	point := (self coordinates) copy.
	xcoor := (point x) copy.
	ycoor := (point y) copy.

	(finalMessage isNil) ifTrue:[
		point setX: (xcoor + ((-1 to: 1) atRandom)) setY: (ycoor + ((-1 to: 1) atRandom)).

		[self location inBounds: point] whileFalse: [
			point setX: (xcoor + ((-1 to: 1) atRandom)) setY: (ycoor + ((-1 to: 1) atRandom)).
		].

		message := MovePersonMessage new.
		message location: self location.
		message coordinates: point.
		message owner: self.
		message time: 1.

		GameEngine instance fireGameMessage: message.
	].! !

!Person methodsFor: 'actions' stamp: 'stuff 12/2/2004 03:27'!
setAction: inAction
	"Removes the old action and adds the new one"

	"Do nothing more"
	(inAction isKindOf: DoNothing) ifTrue: [
		currentAction := inAction.
		"Tell the UI to update:"
		self location ifNotNil: [
			(self location findObject: self) updateAction: inAction.
		].

		^self.
	].

	"already do something"
	(currentAction isNil or: [currentAction isKindOf: DoNothing]) ifFalse:
	[
		^ self.
	].
	
	Transcript show: 'setAction: Get order to ', inAction printString; cr.

	currentAction _ inAction.

	finalMessage _ nil.
	destinationCoordinates _ nil.

	"Tell the UI to update:"
	self location ifNotNil: [
		(self location findObject: self) updateAction: inAction.
	].

	(inAction isKindOf: DoNothing) ifTrue: [
		^self.
	] ifFalse: [
		self doCurrentAction	.
	].! !


!Person methodsFor: 'as yet unclassified' stamp: 'dqn 11/22/2004 16:29'!
initialize
	"Initializes the class variables (in this case, factors, the Dictionary)"

	super initialize.					"Initialize inherited objects"
	factors := Dictionary new.		"Used for storing game factors such as health"
	items   := Bag new.				"Stores the person's current items"
	personalEffects := Bag new.		"Stores the person's personal effects"
	currentAction := DoNothing new.	"At the beginning, people don't really do much."

	NextCoordinate _ OrderedCollection new.
	NextCoordinate add: 0@-1.
  	NextCoordinate add: 0@1.
  	NextCoordinate add: -1@0.
  	NextCoordinate add: 1@0.
	NextCoordinate add: -1@-1.
  	NextCoordinate add: 1@-1.
  	NextCoordinate add: 1@1.
  	NextCoordinate add: -1@1.
! !

!Person methodsFor: 'as yet unclassified' stamp: 'VU 11/6/2004 15:39'!
viewTextRepresentation
	"Returns an array of strings that can be displayed to screen to give information about the WorldObject"

	| retArray |

	retArray := OrderedCollection new.

	retArray add: ('Name: ', self name).
	(self gender = #male) ifTrue: [retArray add: ('Gender: male').].
	(self gender = #female) ifTrue: [retArray add: ('Gender: female').].
	retArray add: ('Coordinates: ', self coordinates asString).

	retArray add: ''.

	retArray add: 'Currently doing: ', self currentAction name.

	retArray add: ''.
	
	"adding attributes"	
	retArray add: 'Attributes: '.
	attributes associationsDo: [:assoc |
		retArray add: assoc asString.
	].

	retArray add: ''.

	"adding Factors"	
	retArray add: 'Factors: '.
	factors associationsDo: [:assoc |
		retArray add: assoc asString.
	].

	^retArray.! !


!Person methodsFor: 'effects' stamp: 'ACK 10/22/2004 19:03'!
addPersonalEffect: effect
	"add a new effect to effect list"

	| effectClasses |

	(effect isKindOf: Effect) 
		ifFalse: [^'Error: I cant magically be effected by something that isnt an effect you nub'].

	personalEffects add: effect.

	effectClasses := effect effects.

	effectClasses associationsDo: [:assoc |
		( (self factor: (assoc key)) isKindOf: Number ) 
			ifFalse: [self factor: (assoc key) is: (assoc value).]
			ifTrue: [self factor: (assoc key) is: ((self factor: (assoc key)) + (assoc value)).].
	].! !

!Person methodsFor: 'effects' stamp: 'ACK 10/22/2004 19:04'!
removePersonalEffect: effect
	"remove an effect from the effect list"
	
	| effectClasses |

	(personalEffects includes: effect) 
		ifFalse: [^'Error: While the idea of removing something I dont have is absolutely adorable, I cant do it'].

	personalEffects remove: effect.

	effectClasses := effect effects.

	effectClasses associationsDo: [:assoc |
		( (self factor: (assoc key)) isKindOf: Number ) 
			ifTrue: [self factor: (assoc key) is: ((self factor: (assoc key)) - (assoc value)).].
	].! !


!Person methodsFor: 'items' stamp: 'dqn 10/30/2004 19:59'!
addItem: anItem
	"just for testing purpose"
	items add: anItem.
	! !

!Person methodsFor: 'items' stamp: 'VU 11/1/2004 16:22'!
dropItem: item
	"Drop the item passed in with item"

	| effectClasses point |

	(items includes: item) ifFalse: 
		[^'Error: While Id love to drop such a lovely item, it surely does not exist in my bag'].
	(location = nil) ifTrue:
		[^'Error: I cannot drop an item into the abyss, Id rather keep it until Im in a proper room'].

	point := location findEmptySpotAround: coordinates x and: coordinates y.
	point ifNotNil: [
		items remove: item.
		effectClasses := item listEffects.
		effectClasses do: [:effect |
			self removePersonalEffect: effect. ].
		location addWorldObject: item at: point x at: point y.
	].! !

!Person methodsFor: 'items' stamp: 'ACK 10/22/2004 19:02'!
hasItem: item
	"Return the Person's items"

	^(items includes: item).	! !

!Person methodsFor: 'items' stamp: 'VU 11/6/2004 15:09'!
hasKey
  "Returns true if the person has a key, and false if not."

  items do: [ :item |
    (item isKindOf: Key) ifTrue: [ ^true. ].
  ].

  ^ false.! !

!Person methodsFor: 'items' stamp: 'VU 12/2/2004 01:17'!
pickupItem: aWorldObject
	"pick up an item in the current location"

	| effectClasses |
	(self nextTo: (aWorldObject coordinates)) ifFalse:
	[
		Transcript show: 'Running to ', aWorldObject printString; cr.
		self runTo: (aWorldObject coordinates).
		^ self.
	].

	"we may need error checking here"
	(items includes: aWorldObject) ifFalse:
	[
		location removeWorldObject: aWorldObject.

		items add: aWorldObject.

		effectClasses := aWorldObject listEffects.

		effectClasses do: [:effect |
			self addPersonalEffect: effect].
	].

	destinationCoordinates _ nil.
	finalMessage _ nil.
	self setAction: DoNothing new.
	! !


!Person methodsFor: 'location' stamp: 'stuff 12/2/2004 03:31'!
moveToLocation: newLocation
  "This now just calls changeLocation and returns the current location... the other functionallity described was from the design for milestone 2.....This method will see if the person can move from the room he is currently in into the new room passed in, and if so, will perform the move. Otherwise, no move will be performed. Either way, the room that the person is in after the function is called is returned."


	"Remove self from the old location if necessary"
	self setAction: DoNothing new.
	location ifNotNil: [ location removeWorldObject: self. ].
	finalMessage _ nil.
	destinationCoordinates _ nil.
	Transcript show: 'moveToLocation'; cr.
	path _ nil.
	newLocation enter: self from: self location.! !


!Person methodsFor: 'modifiers' stamp: 'VU 12/2/2004 01:15'!
changeLocation: newLocation
	"change to new location"

	location := newLocation.
	finalMessage _ nil.
	destinationCoordinates _ nil.
	path _ nil.
	self setAction: DoNothing new.! !

!Person methodsFor: 'modifiers' stamp: 'ACK 10/22/2004 18:57'!
factor: newFactor is: newValue
	"Create/Modify the game factor newFactor to newValue.  If the factor does not exist,
	 it is created.  If it does, it is simply rewritten."

	(newValue isKindOf: Number) ifTrue:
		[factors at: newFactor put: newValue.
		^self].

	^'Error: Factor accepts only Numbers'.! !

!Person methodsFor: 'modifiers' stamp: 'ACK 10/22/2004 18:56'!
gender: newGender
	"Set the Person's gender.  This returns an Error if a value other than the symbols
	 #male or #female are sent in."

	((newGender = #male) or: [newGender = #female]) ifTrue: 
		[gender := newGender.
		 ^self.].

	^'Error: Not a Gender'.! !

!Person methodsFor: 'modifiers' stamp: 'ACK 10/22/2004 18:55'!
name: newName
	"Set a new name"
	
	name := newName.! !


!Person methodsFor: 'xml' stamp: 'ACK 10/29/2004 19:22'!
tagName
  ^ 'person'.! !

!Person methodsFor: 'xml' stamp: 'dqn 10/31/2004 20:00'!
writeXML: xmlWriter
	"Writes an XML representation of a Person. Refer to comments in the code for specifics"

	xmlWriter startTag: self tagName. "open tag Person"
     xmlWriter attribute: 'name' value: (self name asString). 
     xmlWriter attribute: 'gender' value: (self gender asString).
	xmlWriter endTag. "Close the tag with a >"

	self writeCoordinates: xmlWriter.

	"Write Attributes"
     xmlWriter startTag: 'attributes'; endTag.
     WorldObject xmlizeDictionary: attributes using: xmlWriter tagged: 'attribute'.
     xmlWriter endTag: 'attributes'.

	"Write Factors"
     xmlWriter startTag: 'factors'; endTag.
     WorldObject xmlizeDictionary: factors using: xmlWriter tagged: 'factor'.
     xmlWriter endTag: 'factors'.

	"Write items"
	xmlWriter startTag: 'itemBag'; endTag.
	WorldObject xmlizeBag: items using: xmlWriter tagged: 'items'.
	xmlWriter endTag: 'itemBag'.

	"Write personal effects"
"	xmlWriter startTag: 'personalEffects'; endTag.
	WorldObject xmlizeBag: personalEffects using: xmlWriter tagged: 'personalEffect'.
	xmlWriter endTag: 'personalEffects'.
"	
	"Write effects"
	xmlWriter startTag: 'effectBag'; endTag.
	WorldObject xmlizeBag: effectBag using: xmlWriter tagged: 'effect'.
	xmlWriter endTag: 'effectBag'.	
	
	xmlWriter endTag: self tagName. "Close the tag </Person> and return"
	! !


!Person methodsFor: 'move' stamp: 'JMC 12/1/2004 23:16'!
createMatrix
	| matrix |

	matrix _ Array2D width: location width height: location height.
  	(1 to: location height) do: [ :y |
     	(1 to: location width) do: [ :x |
     		(location isWalkable: (x@y)) ifTrue: [
				matrix at: x at: y put: 0.
		     ] ifFalse: [
				matrix at: x at: y put: 1.			
 			].
		]
	].
	matrix at: destinationCoordinates x at: destinationCoordinates y put: 0.
	^ matrix.! !

!Person methodsFor: 'move' stamp: 'dqn 11/22/2004 21:02'!
findPath
	"Find path from coordinates to destinationCoordinates"
	|q f l current next matrix found|

	"reset current path"
	path _ nil.
	last _ 0.
	
	"initialize a queue"
	q _ Array new: (location width * location height + 1).
	f _ 1.
	l _ 1.
	q at: f put: coordinates.
	found _ false.
	matrix _ self createMatrix.

	"BFS"
	[f <= l and: [found = false]] whileTrue:
	[
		current _ q at: f.
		f _ f + 1.
		(1 to: 8) do:
		[
			: i|
			next _ current + (NextCoordinate at: i).
			((location inBounds: next) and: [(matrix at: (next x) at: (next y)) = 0]) ifTrue:
			[
				matrix at: (next x) at: (next y) put: current.
				l _ l + 1.
				q at: l put: next.
				(next = destinationCoordinates) ifTrue:
				[
					found _ true.
				]
			].
		].	
	].
	
	"if there is a way"
	found ifTrue:
	[
		current _ destinationCoordinates.
		path _ Array new: (location width * location height).
		last _ 0.
		[current = coordinates] whileFalse:
		[
			last _ last + 1.
			path at: last put: current.
			current _ matrix at: (current x) at: (current y).
		].
	]
	ifFalse:
	[
		destinationCoordinates _ nil.
		finalMessage _ nil.
	].! !

!Person methodsFor: 'move' stamp: 'stuff 12/2/2004 03:26'!
runTo: position 
	"This method will move the person to position. If position is different  
	from destinationCoordinates, current destination, the person will need to 
	find a new path. If position is destinationCoordinates, no need to find a 
	different path. In case the person has not reached  
	destinationCoordinates, the person will move to the next spot and fire a  
	message to make the Kernel move him next cycle"

	|message |
	"Check to see there is a change in destination"
	(destinationCoordinates isNil" or: [(position = destinationCoordinates) not]") ifTrue:
	[
		destinationCoordinates _ position.
		self findPath.
		pathTry _ 0.
	].	

	"if the person is doing something else, he can be ordered to do other thing"
	(position = destinationCoordinates ) ifFalse: 
	[
"		destinationCoordinates _ position.
		finalMessage _ nil.
		currentAction _ nil.
		self findPath.
		pathTry _ 0.
"		^ self.
	].

	"Check to see if the Person has reached his destination"
	(destinationCoordinates = coordinates) ifTrue:
	[
		finalMessage _ nil.
		destinationCoordinates _ nil.
		(currentAction isKindOf: DoNothing) ifFalse:
		[
			currentAction doAction: self.
		].
		self setAction: DoNothing new.
		^ self.
	].

	"To combine movement with interacting to other stuff like picking item,
	need to check if the person is next to the stuff"

	(self nextTo: position)  ifTrue:
	[
		(finalMessage isNil not and: [(finalMessage isKindOf: MovePersonMessage) not]) ifTrue:
		[
			finalMessage process.
			Transcript show: 'Done processing message'; cr.
			finalMessage _ nil.
			destinationCoordinates _ nil.
			^ self.
		].
		(currentAction isNil) ifFalse:
		[
			"currentAction doAction: self."
			self doCurrentAction.
			(currentAction isKindOf: DoNothing) ifFalse:
			[
				destinationCoordinates _ nil.
				finalMessage _ nil.
				self setAction: DoNothing new.
				^ self.
			].
		].
	].
	"Check if the next spot is valid: no other object on the way"
	((path notNil) and: [location inBounds: (path at: last)]) ifFalse:
	[
		^ self.
	].
	(( path isNil) or: [location isWalkable: (path at: last)]) ifFalse:
	[
		self findPath.
	].

	"If next spot is valid, move his lazy ass "
	(path isNil) ifFalse:
	[
		"Since he marked the next spot nonWalkable to prevent others from
		running into his way, he needs to set it to Walkable before moving"
		Transcript show: name, '->', (path at: last) printString; cr.
		(location inBounds: (path at: last)) ifFalse:
		[
			^ self.
		].
		location moveMapObject: self to: (path at: last).
		last _ last - 1.
	]
	ifTrue:
	[
		pathTry _ pathTry + 1.
		(pathTry = 5) ifTrue:
		[
			Transcript show: 'Some fuckers are blocking the path. Kick their ass out first. Try: ', pathTry printString.
			^ self.
		].
	].
	
	"Fire message to the Kernel. Waiting for Simulator"
	message _ RunPersonMessage new.
	message location: location.
	message coordinates: position.
	message owner: self.
	message time: 1.

	GameEngine instance fireGameMessage: message.

	"Need to mark the next spot as not Walkable, so Kernel doesnt put 2 persons
      on the same spot. This prevent the case that 2 paths cross"
"	(last > 0) ifTrue:
	[
		location setWalkable: (path at: last) isWalkable: false.
	].
"
	
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Person class
	instanceVariableNames: ''!

!Person class methodsFor: 'as yet unclassified' stamp: 'ACK 10/22/2004 19:27'!
new
	"create and instance"

	^super new initialize.! !
