WorldObject subclass: #Portal
	instanceVariableNames: 'location alignment locked tagName locationID '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-Model'!
!Portal commentStamp: 'ACK 11/5/2004 02:12' prior: 0!
This is the portion of the model that lets the Person move from one location to another.  Portals are transport only in one directly.  It can be locked to prevent passage.!


!Portal methodsFor: 'accessors' stamp: 'ACK 10/30/2004 21:15'!
alignment
	"retuns the alignment of this portal... i.e. #north #south #east #west"

	^alignment.

! !

!Portal methodsFor: 'accessors' stamp: 'ACK 10/30/2004 21:12'!
location
	"Returns the location that this portal points to"

	^location.

	! !

!Portal methodsFor: 'accessors' stamp: 'dqn 11/1/2004 00:22'!
locationID
	"Returns the location that this portal points to"

	^locationID.

	! !

!Portal methodsFor: 'accessors' stamp: 'ACK 10/30/2004 22:14'!
locked
	"returns state of lock"

	^locked.

! !

!Portal methodsFor: 'accessors' stamp: 'dqn 10/31/2004 22:25'!
tagName
	^tagName.! !


!Portal methodsFor: 'creation' stamp: 'dqn 10/31/2004 22:28'!
initialize
	"initialize the portal"

	super initialize. 
	locked := false.
	alignment _ 0.
	tagName _ 'portal'.	! !


!Portal methodsFor: 'modifiers' stamp: 'ACK 10/30/2004 21:16'!
alignment: direction
	"sets the alignment direction of this portal"

	alignment := direction.
	
	! !

!Portal methodsFor: 'modifiers' stamp: 'ACK 10/30/2004 21:13'!
location: newLoc
	"sets the location this portal points to"

	location := newLoc.



	! !

!Portal methodsFor: 'modifiers' stamp: 'dqn 11/1/2004 00:21'!
locationID: id
	locationID _ id.! !

!Portal methodsFor: 'modifiers' stamp: 'ACK 10/30/2004 22:15'!
locked: bool
	"set state of door lock"

	locked := bool.



	! !


!Portal methodsFor: 'xml' stamp: 'dqn 11/1/2004 00:02'!
writeXML: xmlWriter
	xmlWriter startTag: self tagName.
	xmlWriter attribute: 'alignment' value: alignment asString.
	xmlWriter attribute: 'locked' value: locked asString.
	xmlWriter attribute: 'toLocation' value: location locationID asString.
	xmlWriter endTag.
	self writeCoordinates: xmlWriter.
	xmlWriter endTag: self tagName.! !


!Portal methodsFor: 'actions' stamp: 'VU 11/7/2004 12:56'!
interactionsFor: aWorldObject
	"returning a bag of actions that can be performed "

	| availableActions changeView personHasKey |

	availableActions := Bag new.

	(aWorldObject isKindOf: Person) ifTrue: [
		personHasKey := aWorldObject hasKey.

		"We let people through only if the door isn't locked or if the person has a key."
		((self locked not) or: [personHasKey]) ifTrue: [
			availableActions add: (ExitAction new portal: self).
		].

		"If the person has a key, then he can lock/unlock doors."
		personHasKey ifTrue: [
			self locked ifTrue: [
				availableActions add: (ActionUnlockDoor new portal: self).
			] ifFalse: [
				availableActions add: (ActionLockDoor new portal: self).
			].
		].
	] ifFalse: [
		"Allow the user to change the view to that location:"
		changeView := ChangeViewMessage new.
		changeView time: 0.
		changeView location: self location.
		availableActions add: changeView.
	].
	
	^availableActions.! !

!Portal methodsFor: 'actions' stamp: 'VU 11/1/2004 15:53'!
name
  "Returns the name of the portal."

  ^ 'to ', location name, (locked ifTrue: [' - locked'] ifFalse: ['']).! !

!Portal methodsFor: 'actions' stamp: 'VU 11/6/2004 18:16'!
viewTextRepresentation
	"Returns an array of strings that can be displayed to screen to give information about the Portal"

	"This method should be implemented in all world object that can be represented on screen"

	| retArray |

	retArray := OrderedCollection new.

	retArray add: 'Portal to the ', location name,
                      '. It is ', (self locked ifTrue: ['locked'] ifFalse: ['unlocked']), '.'.

	^retArray.
	
		

! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Portal class
	instanceVariableNames: ''!

!Portal class methodsFor: 'as yet unclassified' stamp: 'ACK 10/30/2004 21:11'!
new
	"create"

	^super new initialize.

! !
