Object subclass: #WorldObject
	instanceVariableNames: 'attributes effectBag coordinates oldCoordinates '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-Model'!
!WorldObject commentStamp: 'ACK 11/5/2004 02:07' prior: 0!
The WorldObject class is the class from which all others are instantiated.

It acts as the general "thing" in the World. Every physical object in the world is an extension of WorldObject.

WorldObjects should not be instantiated, the idea is to be an abstract class from which others are generated.!


!WorldObject methodsFor: 'accessors' stamp: 'VU 11/6/2004 14:31'!
attribute: attrib
     "Return the value of the attribute at attrib.  Returns Error if non-existant."

     "Check the existance of the Key and error otherwise"
     (attributes includesKey: attrib) ifFalse:
          [^0.].
     
     ^attributes at: attrib.! !

!WorldObject methodsFor: 'accessors' stamp: 'VU 10/30/2004 19:01'!
coordinates
  "Returns the coordinates of the world object in the location"
  ^ coordinates.! !

!WorldObject methodsFor: 'accessors' stamp: 'VU 10/31/2004 00:34'!
coordinates: aPoint
  "Sets the coordinates of the world object to the new value"
	(coordinates isNil) ifTrue:
	[
		oldCoordinates _ aPoint.
	]
	ifFalse:
	[
		oldCoordinates _ coordinates
	].
  coordinates := aPoint.! !

!WorldObject methodsFor: 'accessors' stamp: 'VU 11/6/2004 15:23'!
isCurrentlySelected
	"Returns true if this object is currently slected."

	| co |

	co := GameEngine instance currentPerson.

	^ (co = self).! !


!WorldObject methodsFor: 'as yet unclassified' stamp: 'VU 11/1/2004 11:14'!
viewTextRepresentation
	"Returns an array of strings that can be displayed to screen to give information about the WorldObject"

	"This method should be implemented in all world object that can be represented on screen"

	| errorMessage retArray |

	errorMessage := 'viewTextRepresentation was not implemented.'.
	retArray := OrderedCollection new.

	retArray add: errorMessage.

	^retArray.
	
		

! !


!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:43'!
addEffect: effect
  "This method adds an effect to the world object, and returns the effect added to the object if it was added correctly, or nil if it was not. This is one of those effects that will be affecting people. So, for instance, if the actual instance on which this method is called is a fixed thing, then this method will add some effect that the thing will have on the people in the room."

  (effect isNil) ifTrue: [^nil]. "Cannot add nil effects"
  (effect isKindOf: Effect) ifFalse: [^nil]. "They should actually *be* effects"

  effectBag add: effect.

  ^effect.! !

!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:44'!
attributeCount
  "Responds with the number of attributes associated with the world object."
  ^ attributes size.! !

!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:44'!
effectCount
  "Responds with the number of effects associated with the world object."
  ^ effectBag size.! !

!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:46'!
effects
  ^ self listEffects.

  "TODO: Deprecate this function, because it sucks."! !

!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:45'!
listEffects
  "This returns the collection of all effects that the world object has on people."

  ^ effectBag.
! !

!WorldObject methodsFor: 'effetcts' stamp: 'ACK 10/22/2004 18:47'!
removeEffect: effect
  "Removes the effect from the effect list of the world object, and returns that effect if it was properly removed, or nil if it was not removed or not present in the world object in the first place."

  ^ effectBag remove: effect ifAbsent: [nil.].! !


!WorldObject methodsFor: 'initialization' stamp: 'VU 10/30/2004 19:02'!
initialize
  

	"Initialize the necessary variables"

	attributes := Dictionary new.
	effectBag := Bag new.
	coordinates := (0@0).! !


!WorldObject methodsFor: 'interactions' stamp: 'VU 11/7/2004 16:27'!
interactionsFor: inModel
	"Returns the interactions that are available with the passed in model. This should be implemented in more specific sub-classes."

	^self subclassResponsibility.


			! !


!WorldObject methodsFor: 'modifiers' stamp: 'VU 11/7/2004 19:28'!
applyAttributes: aDictionary
	"Applies all attributes from the dictionary to the object."

	aDictionary keysAndValuesDo: [ :key :value | | cv |
		cv := self attribute: key.
		self attribute: key is: cv + value.
	].! !

!WorldObject methodsFor: 'modifiers' stamp: 'ACK 10/22/2004 18:51'!
attribute: newAttribute is: newValue
     "This allows the creation/modification of an attribute and its newValue.  If the value is
      non-existant, it is created otherwise it is modified to store newValue"

     attributes at: newAttribute put: newValue.! !


!WorldObject methodsFor: 'xml' stamp: 'dqn 10/31/2004 20:13'!
writeCoordinates: xmlWriter
	|x y|

	(oldCoordinates isNil) ifFalse:
	[
		x _ oldCoordinates x.
		y _ oldCoordinates y.
	]
	ifTrue:
	[
		x _ 0.
		y _ 0.
	].
     xmlWriter startTag: 'oldCoordinates'.
	xmlWriter attribute: 'x' value: (x asString).
	xmlWriter attribute: 'y' value: (y asString).
	xmlWriter endTag.
     xmlWriter endTag: 'oldCoordinates'.

	(coordinates isNil) ifFalse:
	[
		x _ coordinates x.
		y _ coordinates y.
	]
	ifTrue:
	[
		x _ 0.
		y _ 0.
	].
     xmlWriter startTag: 'coordinates'.
	xmlWriter attribute: 'x' value: (x asString).
	xmlWriter attribute: 'y' value: (y asString).
	xmlWriter endTag.
     xmlWriter endTag: 'coordinates'.
! !

!WorldObject methodsFor: 'xml' stamp: 'ACK 10/29/2004 18:45'!
writeXML: xmlWriter
     "This is an empty class that will basically give out spurious output if
          called. It is supposed to be overridden"
     
  self subclassResponsibility.
! !


!WorldObject methodsFor: 'move' stamp: 'dqn 11/15/2004 16:15'!
nextTo: aCoordinates
	^ (((coordinates x - (aCoordinates x)) abs < 2 and: [(coordinates y - (aCoordinates y)) abs < 2])).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

WorldObject class
	instanceVariableNames: ''!

!WorldObject class methodsFor: 'as yet unclassified' stamp: 'ACK 10/22/2004 18:48'!
new
     "Initialize the WorldObject"

     
  ^ super new initialize.! !

!WorldObject class methodsFor: 'as yet unclassified' stamp: 'ACK 10/29/2004 23:46'!
xmlizeBag: aBag using: anXMLWriter tagged: aTagName
  "Given a bag, this message will output the contents of the bag utilizing the XMLWriter, and each entry in XML will be named aTagName, and will then contain whatever is the actual value in the bag."

  | executeBlock |

  executeBlock :=
    [:item |
      anXMLWriter startTag: aTagName asString; endTag.

      (item isKindOf: WorldObject)
        ifTrue: [ item writeXML: anXMLWriter. ]
        ifFalse: [ anXMLWriter cdata: item asString. ].

      anXMLWriter endTag: aTagName asString.
    ].

  aBag do: executeBlock.
! !

!WorldObject class methodsFor: 'as yet unclassified' stamp: 'ACK 10/29/2004 23:46'!
xmlizeDictionary: aDictionary using: anXMLWriter tagged: aTagName
  "Given a dictionary, this message will output the contents of the dictionary utilizing the XMLWriter, and each entry in XML will be named the same as the tag name says, and will contain key and value attributes which will correspond to the key-value paring in the dictionary."

  | executeBlock |

  executeBlock :=
    [:key :value |
      anXMLWriter startTag: aTagName asString;
                        attribute: 'key' value: key asString;
                        attribute: 'value' value: value asString;
                      endEmptyTag: aTagName asString.
    ].

  aDictionary keysAndValuesDo: executeBlock.! !
