Object subclass: #FramesetManager
	instanceVariableNames: 'framesets '
	classVariableNames: 'Instance '
	poolDictionaries: ''
	category: 'SimTech-ResourceManagement'!
!FramesetManager commentStamp: 'VU 10/29/2004 19:05' prior: 0!
The frameset manager singleton class is responsible for storing and managing framesets efficiently. It will load framesets upon request, and will hand out references to the framesets to people. Note that the users of the framesets handed out by this manager should not mess with the framesets, because this class will give out references to the objects, and not copy them, for memory considerations. Hence, once loaded, a frameset is immutable whether by contract or through the lack of an appropriate modifying interface.!


!FramesetManager methodsFor: 'initialization' stamp: 'VU 11/1/2004 12:47'!
initialize
  " This method will clean out and initialize the frameset manager. So, if for some reason we want to flush all the entries from the manager, then this message could be sent to the instance of the manager to achieve that."

  framesets := Dictionary new.! !


!FramesetManager methodsFor: 'framesets' stamp: 'VU 12/2/2004 01:31'!
framesetFor: aModel
  "Returns an appropriate graphical representation for the model passed in."
   "TODO DIRTY HACK> :(:(:("

  | frameset frame |

  (aModel isKindOf: Person) ifTrue: [ ^self loadFrameset: 'puyan.gif'. ].
  ((aModel isKindOf: Portal) and: [(aModel isKindOf: Building) not]) ifTrue: [
	frameset := self loadFrameset: 'door.gif'.
	frame := frameset frame: (aModel alignment).
	frameset := SingleFrameSet new.
	frameset setFrame: frame.
	^ frameset.
  ].

  frameset := self loadFrameset: (aModel class name asLowercase), '.gif'.
  frameset ifNotNil: [ ^frameset. ].

  ^self loadFrameset: 'item.gif'.! !

!FramesetManager methodsFor: 'framesets' stamp: 'VU 11/7/2004 15:56'!
loadFrameset: framesetName
  "Loads the frameset with the name passed in if needed. Otherwise, returns the preloaded frameset."

  | fs filePath rawImage imageDir |

  " Check if the frameset is already loaded: "
  fs := framesets at: framesetName ifAbsent: [ nil ].
  fs isNil ifFalse: [ ^ fs ].

  " Apparently, we have not loaded this frameset yet. Well, load it now. "
  imageDir := PathManager instance imagePath.

  (imageDir fileExists: framesetName) ifFalse: [ ^nil ].

  filePath := (imageDir fileNamed: framesetName) fullName.
  rawImage := Form fromFileNamed: filePath.

  rawImage ifNil: [ ^nil. ].

  fs := MarkupFrameSet makeOn: rawImage.

  framesets at: framesetName put: fs.

  ^ fs.! !

!FramesetManager methodsFor: 'framesets' stamp: 'VU 10/30/2004 19:05'!
personFrameset
  "Returns a frameset for a person"! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FramesetManager class
	instanceVariableNames: ''!

!FramesetManager class methodsFor: 'singleton' stamp: 'VU 10/29/2004 19:24'!
instance
  "Returns the one and only instance of this frameset manager class."

  Instance isNil ifTrue: [ Instance := FramesetManager new initialize. ].

  ^ Instance.! !
