FileList subclass: #FileChooser
	instanceVariableNames: ''
	classVariableNames: 'Caller ParentModel SelectedFile Suffix '
	poolDictionaries: ''
	category: 'SimTech-Utilities'!
!FileChooser commentStamp: 'dqn 11/7/2004 00:36' prior: 0!
Will popup a window that allows player to choose and open a saved game from file. The default extension is .sav, which canbe changed.
FileChooser overwrites FileList to give player the only action that he can perform on a file, which is load.
!


!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/4/2004 23:40'!
directory: dir 
	"Set the path of the volume to be displayed."
	self okToChange
		ifFalse: [^ self].
	self modelSleep.
	directory _ dir.
	self modelWakeUp.
	sortMode == nil
		ifTrue: [sortMode _ #date].
	volList _ (Array with: '[]')
				, directory pathParts
				withIndexCollect: [:each :i | "Nesting suggestion from RvL"
					(String new: i - 1 withAll: $ )
						, each].
	self changed: #relabel.
	self changed: #volumeList.
	self pattern: Suffix! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 02:35'!
fileSelectedMenu: aMenu 
	"list of actions that can be performed on the selected files by the player"
	| firstItems services |
	firstItems _ self itemsForFile: self fullName.
	services _ firstItems.
	services
		do: [:svc | svc addDependent: self].
	^ aMenu
		addServices2: services
		for: self
		extraLines: firstItems! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/4/2004 22:36'!
initialize
	super initialize.
! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/4/2004 23:03'!
itemsForFile: fullName 
	"Answer a list of services appropriate for a file of the given full name"
	| aMenu |
	aMenu _ OrderedCollection new: 1.
	aMenu add: self serviceLoadGame.
	^ aMenu! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 02:35'!
itemsForNoFile
	"list of actions that can be performed if no file is being selected"
	| services |
	services _ OrderedCollection new: 3.
	services add: self serviceSortByName.
	services add: self serviceSortBySize.
	services
		add: (self serviceSortByDate).
	^ services! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 02:36'!
loadGame
	"load game from selected file, close FileChooser"
	SelectedFile _ self fullName.
	ParentModel hide.
	ParentModel _ nil.
	Caller processFileName: self fullName! !

!FileChooser methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 02:37'!
serviceLoadGame
	"return a service to load game"
	^ SimpleServiceEntry
		provider: self
		label: 'load game'
		selector: #loadGame
		description: 'load Game'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FileChooser class
	instanceVariableNames: ''!

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 16:43'!
caller
	^ Caller.! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 03:09'!
caller: aCaller
	"who is using FileChooser's service"
	Caller _ aCaller.! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'VU 11/7/2004 16:50'!
open
	"Open a FileChooser"
	ParentModel _
		(FileChooser openAsMorphOn: PathManager instance savePath) openInWorld.! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'VU 11/7/2004 16:49'!
openAsMorphOn: dir
	"Open a morphic view of a FileList on the default directory."
	| aFileList window upperFraction offset |
	aFileList _ self new directory: dir.
	window _ (SystemWindow labelled: dir pathName)
				model: aFileList.
	upperFraction _ 0.3.
	offset _ 0.
	self
		addVolumesAndPatternPanesTo: window
		at: (0 @ 0 corner: 0.3 @ upperFraction)
		plus: offset
		forFileList: aFileList.
	self
		addButtonsAndFileListPanesTo: window
		at: (0.3 @ 0 corner: 1.0 @ upperFraction)
		plus: offset
		forFileList: aFileList.
	window
		addMorph: (PluggableTextMorph
				on: aFileList
				text: #contents
				accept: #put:
				readSelection: #contentsSelection
				menu: #fileContentsMenu:shifted:)
		frame: (0 @ 0.3 corner: 1 @ 1).
	^ window! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 03:08'!
selectedFile
	"return selected file"
	^ SelectedFile.! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 16:50'!
selectedFile: aFile
	"change selected file"
	SelectedFile _ aFile.! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 16:50'!
suffix 
	"return suffix that will be displayed"
	^ Suffix! !

!FileChooser class methodsFor: 'as yet unclassified' stamp: 'dqn 11/7/2004 03:08'!
suffix: aSuffix 
	"suffix that will be displayed"
	Suffix _ aSuffix! !
