LayoutPolicy subclass: #IsometricLayout
	instanceVariableNames: 'tileSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-View'!
!IsometricLayout commentStamp: 'VU 10/25/2004 20:45' prior: 0!
Isometric layout allows us to lay out the children of the morph on an isometric grid. Note that the children of the morph *have* to be MapObjects, otherwise there will be errors, because we really don't want non-map objects on our isometric map.!


!IsometricLayout methodsFor: 'accessing' stamp: 'VU 10/25/2004 20:58'!
tileSize
  "Returns a point containing the dimensions of a single tile in pixels."
  ^ tileSize.! !

!IsometricLayout methodsFor: 'accessing' stamp: 'VU 10/25/2004 20:58'!
tileSize: aPoint
  "Sets a the dimensions of a single tile in pixels to the point passed in."
  tileSize := aPoint.! !


!IsometricLayout methodsFor: 'layout' stamp: 'VU 10/28/2004 19:49'!
layout: aMorph in: newBounds
  "Compute the layout for the given morph based on the new bounds"

  | coords |

  "Iterate through all the children:"
  aMorph submorphsDo: [:child |
    "Get the desired map coordinates for the child. This is where the requirement that"
    "we only lay out map objects comes in:"
    coords := child coordinates.

    "Get the actual coordinates that we want the morph at relative to the parent morph:"
    coords := self morphCoordinatesFor: coords withTileDimensions: (self tileSize).

    child position: (coords + aMorph position).
  ].

! !

!IsometricLayout methodsFor: 'layout' stamp: 'VU 10/25/2004 21:06'!
minExtentOf: aMorph in: newBounds
  "Return the minimal size aMorph's children would require given the new bounds"
  "The minimum size really is the size of a single tile."

  ^ self tileSize.! !

!IsometricLayout methodsFor: 'layout' stamp: 'VU 10/28/2004 19:47'!
morphCoordinatesFor: aPoint withTileDimensions: aDimension
  "Given the desired map coordinates of the morph, and the width of the tile, "
  "this will return the real coordinates where the tile should be put assuming"
  "that the map starts at coordinates (0@0) in the top-left corner."

  ^ self subclassResponsibility.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IsometricLayout class
	instanceVariableNames: ''!

!IsometricLayout class methodsFor: 'instance creation' stamp: 'VU 10/25/2004 21:00'!
new
  "Creates a new isometric layout with the default tile size of 32x32 pixels."

  | result |

  result := super new.
  result tileSize: 32@32.

  ^ result.! !
