ImageMorph subclass: #MapObject
	instanceVariableNames: 'coordinates model trimming '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-View'!
!MapObject commentStamp: 'VU 12/1/2004 23:52' prior: 0!
This is the basic map object. Everything in the view is a map object: tiles, people, buildings, everything. The trimming thing is basically a nice visual effect. For instance, if we are talking about a person, the trimming could be a bubble over their head telling use what they are doing currently, or if we are talking about building that is on fire, the trimming would be the fire, etc.!


!MapObject methodsFor: 'accessing' stamp: 'VU 10/26/2004 19:00'!
containsPoint: aPoint
  "Rturns true if the point in the image morph is not a transparent color, and false if it is transparent, so the user cannot see the point and thus cannot click it."

  | testPoint testColor |

  testPoint := aPoint - (self position).

  testColor := self image colorAt: testPoint.

  ^ (testColor = Color white) not.! !

!MapObject methodsFor: 'accessing' stamp: 'VU 10/22/2004 19:30'!
coordinates
  ^coordinates.! !

!MapObject methodsFor: 'accessing' stamp: 'VU 10/22/2004 19:30'!
model
  ^ model.! !

!MapObject methodsFor: 'accessing' stamp: 'VU 12/1/2004 23:51'!
trimming
	"Returns the trimming"
	^trimming.! !


!MapObject methodsFor: 'events' stamp: 'VU 11/18/2004 01:50'!
displayInteractionsFor: clicker on: clickee
  "Displays a list of interactions between the two objects passed in."

  | interactions menu msg title selector |

  clickee isNil ifTrue: [
    interactions := self privateDefaultActions: clicker.
    title := ' '.
  ] ifFalse: [
    interactions := clickee interactionsFor: clicker.
    title := clickee name.
  ].

  "Create the menu:"
  menu := MenuMorph entitled: title.

  interactions do: [ :action |
    (action isKindOf: SimMessage) ifTrue: [
      msg := action.
    ] ifFalse: [
      msg := ChangeActionMessage new.
      msg owner: clicker.
      msg time: 0.
      msg action: action.
    ].

    selector := (msg isKindOf: CriticalMessage)
      ifTrue: [#fireCriticalMessage:]
      ifFalse: [#fireKernelMessage:].

    menu
      add: action name
      target: GameEngine instance
      selector: selector
      argument: msg.
  ].

  menu position: self position.
  menu popUpInWorld: World.

  ^ menu.

! !

!MapObject methodsFor: 'events' stamp: 'VU 10/22/2004 19:31'!
handlesMouseDown: evt
	"we want to handle all mouse clicks"
	^true! !

!MapObject methodsFor: 'events' stamp: 'VU 11/18/2004 11:47'!
mouseDown: evt
  "Handles mouse event"

  evt redButtonPressed ifTrue: [
"    Transcript show: 'Left-clicked map object at ', (self coordinates asString); cr."

    GameEngine instance currentPerson: self model.

    ^ self.
  ].

  evt yellowButtonPressed ifTrue: [
"    Transcript show: 'Right-clicked map object at ', (self coordinates asString); cr."

    self displayInteractionsFor: (GameEngine instance currentPerson) on: self model.

    ^ self.
  ].! !

!MapObject methodsFor: 'events' stamp: 'VU 12/2/2004 00:45'!
updateAction: anAction
	"Updates the UI for the action of self."
	self privateClearTrimmings. "Clear out existing trimmings."

	anAction ifNotNil: [
		(anAction isKindOf: DoNothing) ifFalse: [
			trimming := ((FramesetManager instance framesetFor: anAction) frame: 1) asMorph.
			self addMorph: trimming.
			trimming position: self position - self privateSafeAnchor.
		].
	].
! !


!MapObject methodsFor: 'initialization' stamp: 'VU 10/25/2004 23:30'!
initialize
  "Performs the initialization for a map object"
  super initialize.
  coordinates := (0@0).
  model := nil.
! !


!MapObject methodsFor: 'private' stamp: 'VU 12/1/2004 23:53'!
privateClearTrimmings
	"Clears out the trimming."
	trimming ifNotNil: [
		trimming delete.
		trimming := nil.
	].! !

!MapObject methodsFor: 'private' stamp: 'dqn 12/1/2004 04:42'!
privateDefaultActions: clicker
  "Returns a list of default actions."

  |  result moveMsg personMsg thingMsg saveMsg loadMsg switchMsg buildingMsg saveWorldMsg|

  result := Bag new.

  (clicker isKindOf: Person) ifTrue: [
    moveMsg := MovePersonMessage new.
    moveMsg owner: clicker.
    moveMsg time: 0.
    moveMsg location: clicker location.
    moveMsg coordinates: self coordinates.

    result add: moveMsg.
  ] ifFalse: [
    personMsg := UserCreatePersonMessage new.
    personMsg time: 0.
    personMsg coordinates: self coordinates.
    result add: personMsg.

    thingMsg := UserCreateThingMessage new.
    thingMsg time: 0.
    thingMsg floor: self.
    thingMsg location: GameEngine instance currentLocation.
    result add: thingMsg.

    saveMsg _ SaveGameMessage new.
    saveMsg time: 0.
    result add: saveMsg.

    saveWorldMsg _ SaveWorldMessage new.
    saveWorldMsg time: 0.
    result add: saveWorldMsg.

    loadMsg _ LoadGameMessage new.
    loadMsg time: 0.
    result add: loadMsg.

    switchMsg := SwitchSimulationStateMessage new.
    switchMsg time: 0.
    result add: switchMsg.

    buildingMsg := CreateBuildingMessage new.
    buildingMsg time: 0.
    buildingMsg location: GameEngine instance currentLocation.
    buildingMsg coordinates: self coordinates.
    result add: buildingMsg.
  ].

  ^ result.! !

!MapObject methodsFor: 'private' stamp: 'VU 10/26/2004 18:48'!
privateSafeAnchor
  "This returns the anchor for this map object if it has an anchor, or (0, 0) if it has no anchor."

  | frame result |

  frame := self image.

  (frame isKindOf: Frame)
    ifTrue: [ result := frame anchor. ]
    ifFalse: [ result := (0@0). ].

  ^ result.! !


!MapObject methodsFor: 'modifying' stamp: 'VU 12/2/2004 00:41'!
coordinates: aPoint
  coordinates := aPoint.

"
	trimming ifNotNil: [
		trimming coordinates: aPoint.
	].
"! !

!MapObject methodsFor: 'modifying' stamp: 'VU 10/22/2004 19:30'!
model: aModel
  model := aModel.! !

!MapObject methodsFor: 'modifying' stamp: 'VU 10/26/2004 18:48'!
position: aPoint
  "Sets the position of the map object. We need to override this function in order to make sure that the anchor is preserved."

  super position: (aPoint - self privateSafeAnchor).! !

!MapObject methodsFor: 'modifying' stamp: 'VU 11/17/2004 12:12'!
slideTo: aPoint
	"Given a point containing map coordinates, will slide from the current location to the new location in half a second. Good stuff."

	self coordinates: aPoint.

"
	| startPoint stopPoint frame |

	startPoint := self position.

	self visible: false.
	self coordinates: aPoint.

	stopPoint := self position.
	frame := self image.

	frame slideFrom: startPoint to: stopPoint nSteps: 20 delay: 50.

	self visible: true.
"

"
	self slideFrom: self position to: (owner position + (layout morphCoordinatesFor: aPoint withTileDimensions: layout tileSize)) nSteps: 10 delay: 10.

	self coordinates: aPoint.
"

"
	startPoint := self position.
	owner := self owner.
	self coordinates: aPoint.
	stopPoint 

	self delete.
	self position: startPoint.

	| layout targetPoint offset xInc yInc inc delay oldPosition newPosition |

	layout := owner layoutPolicy.
	oldPosition := self position.
	self coordinates: aPoint.
	newPosition := self position.
	owner layoutPolicy: nil.
	

	targetPoint := layout morphCoordinatesFor: aPoint withTileDimensions: layout tileSize.
	targetPoint := targetPoint + owner position.
	offset := targetPoint - self position.
	xInc := offset x // 10.
	yInc := offset y // 10.
	inc := xInc@yInc.
	delay := Delay forMilliseconds: 10.

	owner layoutPolicy: nil.
	(1 to: 10) do: [ :index |
		self position: self position + inc.
		delay wait.
	].

	self coordinates: aPoint.
	owner layoutPolicy: layout.

"! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MapObject class
	instanceVariableNames: ''!

!MapObject class methodsFor: 'instance creation' stamp: 'VU 10/25/2004 22:19'!
new
  "Creates and initializes a new instance of self."
   ^ super new initialize.! !
