FrameSet subclass: #MarkupFrameSet
	instanceVariableNames: 'frames '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SimTech-View'!
!MarkupFrameSet commentStamp: 'VU 10/18/2004 14:30' prior: 0!
This is a frameset implemented using our own animation format - basically, it's a plain image with some markup delimiting the frames.!


!MarkupFrameSet methodsFor: 'accessing' stamp: 'VU 10/18/2004 16:19'!
frame: index
  "Returns the frame given the index."
  ^ frames at: index.! !

!MarkupFrameSet methodsFor: 'accessing' stamp: 'VU 10/18/2004 16:19'!
frameCount
  "Returns the number of frames in the frame set."
  ^ frames size.! !


!MarkupFrameSet methodsFor: 'initialization' stamp: 'VU 10/18/2004 15:21'!
initialize: aForm
  "Initializes the frameset from the form."

  | width cDelimit cInside cAnchor vBounds hBounds frame |

  "First, get the width of the image."
  width := aForm width.

  "We would like to figure out what the special markup colors are."
  cDelimit := aForm colorAt: (width-1)@0.
  cInside := aForm colorAt: (width-1)@1.
  cAnchor := aForm colorAt: (width-1)@2.

  Transcript show: 'Delimit - ', (cDelimit asString); cr.
  Transcript show: 'Inside - ', (cInside asString); cr.
  Transcript show: 'Anchor - ', (cAnchor asString); cr.

  "Now, find the vertical bounds of the tiles in the image:"
  vBounds := self findVerticalBounds: aForm delimitedBy: cDelimit.

  Transcript show: 'Vertical Bounds - ', (vBounds asString); cr.

  "Now, find the horizontal bounds of the tiles in the image:"
  hBounds := self findHorizontalBounds: aForm delimitedBy: cDelimit.

  Transcript show: 'Horizontal Bounds - ', (hBounds asString); cr.

  "Initialize the frame list:"
  frames := OrderedCollection new.

  "Scan in the frames:"
  hBounds do: [:y |
    vBounds do: [:x |
      frame := self scanFrame: aForm at: (x@y) colored: cInside anchored: cAnchor.
      frames add: frame.
    ].
  ].

  Transcript show: 'Frame Count - ', (frames size asString); cr.! !


!MarkupFrameSet methodsFor: 'private' stamp: 'VU 10/18/2004 15:35'!
findColor: aColor in: aFrame at: aPoint horizontal: walkHorizontally
  "Returns the first occurrence of aColor in aFrame, starting at aPoint, and walking to the right from there if walkHorizontally is true, and down if it is false."

  | x y width height result |

  x := aPoint x.
  y := aPoint y.

  walkHorizontally ifTrue: [
    width := aFrame width.
    [(x < width) and: [((aFrame colorAt: x@y) = aColor) not]] whileTrue: [ x := x + 1. ].
    result := x.
  ] ifFalse: [
    height := aFrame height.
    [(y < height) and: [((aFrame colorAt: x@y) = aColor) not]] whileTrue: [ y := y + 1. ].
    result := y.
  ].

  ^ result.! !

!MarkupFrameSet methodsFor: 'private' stamp: 'VU 10/18/2004 15:07'!
findHorizontalBounds: aForm delimitedBy: aColor
  "Answers with the list of coordinates of horizontal bounds of tiles in the form."

  | result |

  result := OrderedCollection new.

  "The reason it's height-2, is because the pixels start at 0 and not at 1, *and* we don't want to count the last delimiter."
  0 to: ((aForm height) - 2) do: [:index |
    ((aForm colorAt: (0 @ index)) = aColor)
      ifTrue: [ result addLast: index. ].
  ].

  ^ result.! !

!MarkupFrameSet methodsFor: 'private' stamp: 'VU 10/18/2004 15:46'!
findNotColor: aColor in: aFrame at: aPoint horizontal: walkHorizontally
  "Returns the first occurrence of color other than aColor in aFrame, starting at aPoint, and walking to the right from there if walkHorizontally is true, and down if it is false."

  | x y width height result |

  x := aPoint x.
  y := aPoint y.

  walkHorizontally ifTrue: [
    width := aFrame width.
    [(x < width) and: [(aFrame colorAt: x@y) = aColor]] whileTrue: [ x := x + 1. ].
    result := x.
  ] ifFalse: [
    height := aFrame height.
    [(y < height) and: [(aFrame colorAt: x@y) = aColor]] whileTrue: [ y := y + 1. ].
    result := y.
  ].

  ^ (result - 1).! !

!MarkupFrameSet methodsFor: 'private' stamp: 'VU 10/18/2004 15:04'!
findVerticalBounds: aForm delimitedBy: aColor
  "Answers with the list of coordinates of vertical bounds of tiles in the form."

  | result |

  result := OrderedCollection new.

  "The reason it's width-2, is because the pixels start at 0 and not at 1, *and* we don't want to count the last delimiter."
  0 to: ((aForm width) - 2) do: [:index |
    ((aForm colorAt: (index @ 0)) = aColor)
      ifTrue: [ result addLast: index. ].
  ].

  ^ result.! !

!MarkupFrameSet methodsFor: 'private' stamp: 'VU 10/18/2004 16:14'!
scanFrame: aForm at: aPoint colored: insideColor anchored: anchorColor
  "Answers with a frame scanned from aForm starting at aPoint using the insideColor, and anchorColor"

  | x y hAnchor vAnchor width height |

  x := self findColor: anchorColor in: aForm at: aPoint horizontal: true.
  y := self findColor: anchorColor in: aForm at: aPoint horizontal: false.

  hAnchor := x - (aPoint x) - 1.
  vAnchor := y - (aPoint y) - 1.

  x := self findNotColor: insideColor in: aForm at: ((x + 1)@(aPoint y)) horizontal: true.
  y := self findNotColor: insideColor in: aForm at: ((aPoint x)@(y + 1)) horizontal: false.

  width := x - (aPoint x) - 1.
  height := y - (aPoint y) - 1.

  Transcript show: 'Anchor at - ', ((hAnchor@vAnchor) asString); cr.
  Transcript show: 'Frame size - ', ((width@height) asString); cr.

  ^ Frame frameOn: aForm
            bounds: (Rectangle origin: ((aPoint x) + 1)@((aPoint y) + 1)
                                   extent: (width@height))
            anchoredAt: (hAnchor@vAnchor).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MarkupFrameSet class
	instanceVariableNames: ''!

!MarkupFrameSet class methodsFor: 'instance creation' stamp: 'VU 10/18/2004 14:39'!
makeOn: aFrame
  "Given a frame containing an image, essentially parses the image and makes a frameset out of it."

  ^ self new initialize: aFrame.! !
