Object subclass: #UIManager
	instanceVariableNames: 'mapView mainWindow selectedItemMorph '
	classVariableNames: 'Instance '
	poolDictionaries: ''
	category: 'SimTech-View'!

!UIManager methodsFor: 'accessing' stamp: 'VU 10/30/2004 22:12'!
getLocationGrid
  "Returns the location grid that is used for displaying the current view."
  ^ mapView locationGrid.! !

!UIManager methodsFor: 'accessing' stamp: 'VU 10/30/2004 22:19'!
getMainWindow
  "Returns the main window for the game."
  ^ mainWindow.! !

!UIManager methodsFor: 'accessing' stamp: 'VU 10/30/2004 22:11'!
getMapMorph
  "Returns the morph that contains the map. Generally, we won't need to use this function."
  ^mapView.! !

!UIManager methodsFor: 'accessing' stamp: 'VU 10/30/2004 22:13'!
getSelectedItemMorph
  "Returns the selected item morph so that when an item is selected, we can notify it and update it properly."

  ^ selectedItemMorph.! !


!UIManager methodsFor: 'initialization' stamp: 'VU 10/30/2004 22:15'!
initialize
  "Initializes the UI Manager."

  mainWindow := SimTechWindow new.
  selectedItemMorph := SelectedItemMorph new.
  mapView := MapMorph new.

  mainWindow addMorph: selectedItemMorph frame: ((0.8@0) corner: (1@0.2)).
  mainWindow addMorph: mapView frame: ((0@0) corner: (1@1)).
! !

!UIManager methodsFor: 'initialization' stamp: 'VU 11/18/2004 12:04'!
setLocationView: aLocation
	"Sets the location view to the new location."

	| tileSet layout adjustedPosition |

	"Remove the old map view:"
	mapView delete.

	mapView := MapMorph new.
	mainWindow addMorph: mapView frame: ((0@0) corner: (1@1)).

	tileSet := FramesetManager instance loadFrameset: 'floor.gif'.

	layout := DiamondLayout new.
	layout tileSize: 48@23.

	mapView locationGrid layoutPolicy: layout.
	mapView locationGrid tileset: tileSet.
	mapView locationGrid buildFromLocation: aLocation.

	adjustedPosition _ 0@0 - (mapView locationGrid fullBounds origin).

"	Transcript show: 'Setting position to: ', (adjustedPosition asString); cr;show:''."

	UIManager instance getLocationGrid position: adjustedPosition.

! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

UIManager class
	instanceVariableNames: ''!

!UIManager class methodsFor: 'instance creation' stamp: 'VU 10/30/2004 22:08'!
instance
  "Return the one and only instance of the UI manager."

  Instance ifNil: [ self new. ].

  ^ Instance.! !

!UIManager class methodsFor: 'instance creation' stamp: 'VU 10/30/2004 22:07'!
new
  "Shouldn't instantiate UI Manager, man."

  Instance isNil
    ifTrue: [ Instance := super new initialize. ^Instance. ]
    ifFalse: [ ^self error: 'Shouldn''t instantiate UI Manager, man.'. ]. ! !

!UIManager class methodsFor: 'instance creation' stamp: 'VU 11/7/2004 16:51'!
restart
	"Restarts the interface."

	UIManager instance getMainWindow delete.
	UIManager start.! !

!UIManager class methodsFor: 'instance creation' stamp: 'VU 11/6/2004 16:08'!
start
  "Will start up the game, creating all the necessary UI as appropriate. This assumes that the GameEngine has already been initialized."

  | room |

  UIManager instance initialize.

  room := GameEngine instance currentLocation.

  UIManager instance setLocationView: room.

  UIManager instance getSelectedItemMorph currentObject: (GameEngine instance currentPerson).

  UIManager instance getMainWindow openInWorld.! !


!UIManager class methodsFor: 'examples' stamp: 'VU 11/6/2004 14:37'!
example

  | room tileSet layout |

  UIManager instance initialize.
"
  puyanModel := Person new.
  puyanModel name: 'Puyan the Little Guy'.
  puyanModel gender: #male.

  bookModel := Item new.
  bookModel name: 'Book: FreeBSD Networking'.
  bookModel description: 'This is a book on how to do networking in FreeBSD.'.

  room := Location new.
  room width: 16 height: 16.
  room addWorldObject: puyanModel at: 10 at: 10.
  puyanModel changeLocation: room.
  room addWorldObject: bookModel at: 3 at: 3.

  GameEngine instance currentLocation: room.
"
  room _ GameEngine instance currentLocation.

  tileSet := FramesetManager instance loadFrameset: 'floor.gif'.
  FramesetManager instance loadFrameset: 'puyan.gif'.
  FramesetManager instance loadFrameset: 'book.gif'.

  layout := DiamondLayout new.
  layout tileSize: 48@23.

  UIManager instance getLocationGrid layoutPolicy: layout.
  UIManager instance getLocationGrid tileset: tileSet.
  UIManager instance getLocationGrid buildFromLocation: room.
  UIManager instance getLocationGrid position: (0@0 - (UIManager instance getMapMorph scroller submorphBounds origin)).

  UIManager instance getSelectedItemMorph currentObject: (GameEngine instance currentPerson).

  UIManager instance getMainWindow openInWorld.! !
