RectangleMorph subclass: #AbstractDialog
	instanceVariableNames: 'response responseUponCancel content contentSelector done '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Vladimir-Utilities'!

!AbstractDialog methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:06'!
response
	"Returns the current response"
	^ response.! !

!AbstractDialog methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:07'!
response: aText
	"Sent when the user has made his choice."

	response _ aText asString.
	done _ true.
	^ true.! !

!AbstractDialog methodsFor: 'accessing' stamp: 'VU 11/7/2004 13:37'!
responseUponCancel: anObject
	"Sets the response to return when the user clicks cancel."
	responseUponCancel _ anObject
! !

!AbstractDialog methodsFor: 'accessing' stamp: 'VU 11/7/2004 15:29'!
setQuery: queryString
	"Sets up the window."

	| accept cancel buttonAreaHeight query frame topOffset |

	done _ false.
	self removeAllMorphs.

	self layoutPolicy: ProportionalLayout new.

	query _ TextMorph new contents: queryString.
	query setNameTo: 'query'.
	query lock.
		frame _ LayoutFrame new.
		frame topFraction: 0.0; topOffset: 2.
		frame leftFraction: 0.5; leftOffset: (query width // 2) negated.
	query layoutFrame: frame.
	self addMorph: query.
	topOffset _ query height + 4.

	accept _ self createAcceptButton.
	self addMorph: accept.

	cancel _ self createCancelButton.
	self addMorph: cancel.
	buttonAreaHeight _ (accept height max: cancel height) + 4.

	content _ self privateGetContent.
		frame _ LayoutFrame new.
		frame leftFraction: 0.0; rightFraction: 1.0; topFraction: 0.0; topOffset: topOffset; bottomFraction: 1.0; bottomOffset: buttonAreaHeight negated.
	content layoutFrame: frame.
	self addMorph: content.

	self extent:
		(((query extent x max: content extent x) + 10)
		@
		((topOffset + content extent y + 10 + buttonAreaHeight))).! !


!AbstractDialog methodsFor: 'initialization' stamp: 'VU 11/7/2004 13:35'!
createAcceptButton
	"create the [accept] button"

	| result frame |

	result := SimpleButtonMorph new target: self;
				 color: Color lightGreen.

	result
		borderColor: (Preferences menuAppearance3d
				ifTrue: [#raised]
				ifFalse: [result color twiceDarker]).

	result label: 'Accept(s)';
		 actionSelector: #accept.

	result setNameTo: 'accept'.

	frame := LayoutFrame new.
	frame rightFraction: 0.5;
		 rightOffset: -10;
		 bottomFraction: 1.0;
		 bottomOffset: -2.

	result layoutFrame: frame.

	self addMorph: result.
	[ self
		updateColor: result
		color: result color
		intensity: 2 ] on: MessageNotUnderstood do: [ :ex | ].

	^ result! !

!AbstractDialog methodsFor: 'initialization' stamp: 'VU 11/7/2004 13:36'!
createCancelButton
	"create the [cancel] button"

	| result frame |

	result := SimpleButtonMorph new target: self;
				 color: Color lightRed.

	result
		borderColor: (Preferences menuAppearance3d
				ifTrue: [#raised]
				ifFalse: [result color twiceDarker]).

	result label: 'Cancel(l)';
		 actionSelector: #cancel.

	result setNameTo: 'cancel'.

	frame := LayoutFrame new.
	frame leftFraction: 0.5;
		 leftOffset: 10;
		 bottomFraction: 1.0;
		 bottomOffset: -2.

	result layoutFrame: frame.

	self addMorph: result.
	[ self
		updateColor: result
		color: result color
		intensity: 2 ] on: MessageNotUnderstood do: [ :ex | ].

	^ result! !

!AbstractDialog methodsFor: 'initialization' stamp: 'VU 11/7/2004 13:36'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color white! !

!AbstractDialog methodsFor: 'initialization' stamp: 'VU 11/7/2004 14:12'!
initialize
	"initialize the state of the receiver"
	super initialize.

	"Use round corners if the user would like that."
	Preferences roundedWindowCorners
		ifTrue: [self useRoundedCorners].
	
	self extent: 200 @ 70.
	response _ ''.
	responseUponCancel _ ''! !


!AbstractDialog methodsFor: 'invoking' stamp: 'VU 11/7/2004 13:46'!
getUserResponse
	"Wait for the user to accept or cancel, and answer the result string. Answers the empty string if the user cancels."

	"Details: This is invoked synchronously from the caller. In order to keep processing inputs and updating the screen while waiting for the user to respond, this method has its own version of the World's event loop."

	| w |
	w _ self world.
	w ifNil: [^ response].
	done _ false.
	[done] whileFalse: [w doOneCycle].
	self delete.
	w doOneCycle.
	^ response
! !


!AbstractDialog methodsFor: 'menu' stamp: 'VU 11/7/2004 14:10'!
accept
	"Sent by the accept button"
	content perform: self privateGetContentSelector.! !

!AbstractDialog methodsFor: 'menu' stamp: 'VU 11/7/2004 14:43'!
cancel
	"Sent by the cancel button."

	response _ responseUponCancel.
	done _ true.
! !


!AbstractDialog methodsFor: 'submorphs-add/remove' stamp: 'VU 11/7/2004 14:34'!
delete

	self breakDependents.
	super delete.! !


!AbstractDialog methodsFor: 'private' stamp: 'VU 11/7/2004 13:59'!
privateGetContent
	"This function will return the morph that will provide the content for the user to type stuff in."

	^ self subclassResponsibility.! !

!AbstractDialog methodsFor: 'private' stamp: 'VU 11/7/2004 14:00'!
privateGetContentSelector
	"Will respond with the selector that needs to be called on the content morph to get the user slection from it."

	^ self subclassResponsibility.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractDialog class
	instanceVariableNames: ''!

!AbstractDialog class methodsFor: 'instance creation' stamp: 'VU 11/7/2004 13:49'!
request: queryString
	"Pops up the request"

	^ self
		request: queryString
		centerAt: (ActiveHand ifNil:[Sensor]) cursorPoint
		inWorld: ActiveWorld! !

!AbstractDialog class methodsFor: 'instance creation' stamp: 'VU 11/7/2004 14:37'!
request: queryString centerAt: aPoint inWorld: aWorld
	"Creates an instance of the dialog in the world with the specified title, at the specified point"

	| aDialog |

	aDialog := self new.
	aDialog setQuery: queryString.
	aWorld addMorph: aDialog centeredNear: aPoint.

	^ aDialog getUserResponse.
! !
