Object subclass: #FileInOut
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Vladimir-Utilities'!
!FileInOut commentStamp: 'VU 9/25/2004 20:56' prior: 0!
This class will help us file stuff in and file stuff out.

To file in from some directory, use the fileInDirectory: method. It will file in all *.st files from the directory. For example:

FileInOut fileInDirectory: 'C:\CS2340\Milestones\M2\src'.

To file out all classes from a category, use the fileOutCategory:to: method. It will file out all classes in the specified category as .st files, one class per file. For example:

FileInOut fileOutCategory: 'SimTech' to: 'C:\CS2340\Milestones\M2\src'.!


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FileInOut class
	instanceVariableNames: ''!

!FileInOut class methodsFor: 'as yet unclassified' stamp: 'VU 9/26/2004 17:10'!
fileInDirectory: directoryPath
  "Files in all .st files from the directory passed in."

  | dir lstFiles stream |

  Transcript cr; cr.

  "Gets the directory:"
  dir := FileDirectory on: directoryPath.

  "Gets a list of full paths to all .st files:"
  lstFiles := dir fileNames
    select: [:fileName | fileName endsWith: '.st']
      thenCollect: [:fileName | (dir pathName), (dir pathNameDelimiter asString), fileName.].

  "Then files them all in:"
  lstFiles do:
    [:fileName | 
      Transcript show: ('Filing in ', fileName, '...'); tab.

      stream := FileStream fileNamed: fileName.
      stream fileIn.
      stream close.
      Transcript show: 'done'; cr.
    ].! !

!FileInOut class methodsFor: 'as yet unclassified' stamp: 'VU 9/26/2004 18:28'!
fileOutCategory: aCategory to: aDirectory
  "Files out all classes in the category into the directory passed in."

  | so classes dir fileName outputStream |

  so := Smalltalk organization.
  classes := so superclassOrder: aCategory.

  dir := FileDirectory on: aDirectory.
  (dir directoryExists: aCategory) ifFalse: [dir createDirectory: aCategory.].
  dir := dir directoryNamed: aCategory.

  classes withIndexDo: [:value :index |
"    fileName := dir fullNameFor: (index asString, '_', value name, FileDirectory dot, 'st')."
    fileName := (value name, FileDirectory dot, 'st').
    outputStream := dir forceNewFileNamed: fileName.
    fileName := dir fullNameFor: fileName.

    Transcript show: 'Filing out to ', fileName, ' class: '.
    Transcript tab.

    value
          fileOutOn: outputStream
          moveSource: false
          toFile: 0
          initializing: true.

    outputStream flush.
    outputStream close.

    Transcript tab; show: 'done!!'; cr; cr.
  ].
! !

!FileInOut class methodsFor: 'as yet unclassified' stamp: 'VU 10/29/2004 22:42'!
fromDirectory: aDirectory fileInCategories: categoryList
  "Files in all categories in the category list from the directory passed in."

  | dir subdir |

  dir := FileDirectory on: aDirectory.

  dir isNil ifTrue: [ ^ self error: 'Could not locate directory.' ].

  categoryList do: [:item |
    subdir := dir directoryNamed: item.

    subdir isNil ifTrue: [ ^self error: 'Could not locate directory ', item, ' in ', aDirectory. ].

    3 timesRepeat: [
      FileInOut fileInDirectory: subdir fullName.
    ].
  ].! !
