BorderedMorph subclass: #RadioButtonGroupMorph
	instanceVariableNames: 'values acceptTarget acceptSelector target getSelector getArgument setSelector setArgument '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Vladimir-Utilities'!
!RadioButtonGroupMorph commentStamp: 'VU 11/7/2004 13:18' prior: 0!
This is a simple radio button group that takes a single target and sets buttons based on the return value of performing a selector on that target.

See the class side for examples of its use.

initializeWithValues: anArrayOfValues
	this constructs the buttons, and makes them each correspond to a value in anArrayOfValues.

target:
	this sets the object that is queried and set.

getSelector:
getArgument:
	these set the selector and argument that is sent to the target to get the current value.
	
setSelector:
setArgument:
	these set the selector and argument that is sent to the target to set the value when a button is pushed.

Note: This code comes straight from the Squeak Swiki, located at the following URL:
	http://minnow.cc.gatech.edu/squeak/1362
!


!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:57'!
accept
	"Accepts the selection"

	acceptTarget perform: acceptSelector with: self value.! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:53'!
acceptSelector: aSelector
	"Sets the selector for the accept action."
	acceptSelector _ aSelector.! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:52'!
acceptTarget: aTarget
	"Sets the target of the accept action"
	acceptTarget := aTarget.! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'nk 7/25/2002 15:37'!
getArgument: anObject
	getArgument _ anObject
! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'nk 7/25/2002 15:36'!
getSelector: selector
	getSelector _ selector.
! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'nk 7/25/2002 15:53'!
setArgument: anObject
	setArgument _ anObject
! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'nk 7/25/2002 15:53'!
setSelector: selector
	setSelector _ selector.
! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'nk 7/25/2002 15:37'!
target: anObject
	target _ anObject
! !

!RadioButtonGroupMorph methodsFor: 'accessing' stamp: 'VU 11/7/2004 14:56'!
value
	"Returns the currently selected value."
	(getSelector isNil or: [ target isNil ]) ifTrue: [ ^nil ].
	
	^(getArgument
		ifNil: [ target perform: getSelector ]
		ifNotNil: [ target perform: getSelector withArguments: { getArgument } ]).
! !


!RadioButtonGroupMorph methodsFor: 'initialization' stamp: 'VU 11/7/2004 15:27'!
initializeWithValues: arrayOfValues
	"Given arrayOfValues, construct my buttons.
	One of the values should match the return value from performing my getSelector"
	| button |
	values _ arrayOfValues.
	
	self layoutPolicy: TableLayout new;
		hResizing: #spaceFill;
		vResizing: #spaceFill;
		listCentering: #center;
		color: Color transparent;
		borderWidth: 1.

	self addMorphBack: (Morph new color: Color transparent; vResizing: #spaceFill).
	arrayOfValues do: [ :val | | aligner |
		aligner _ (Morph new color: Color transparent;
			layoutPolicy: TableLayout new;
			listDirection: #leftToRight;
			layoutInset: 4@0;
			cellInset: 2@-2;
			hResizing: #spaceFill;
			vResizing: #shrinkWrap).
		button _ UpdatingThreePhaseButtonMorphEnhanced radioButton.
		button target: self;
			actionSelector: #setValue:;
			arguments: { val };
			getSelector: #isValue:;
			getArgument: val.
		aligner addMorphBack: button.
		aligner addMorphBack: (StringMorph contents: val asString).
		self addMorphBack: aligner.
		self addMorphBack: (Morph new color: Color transparent; vResizing: #spaceFill).
	].

	self extent: self fullBounds extent.
	
! !


!RadioButtonGroupMorph methodsFor: 'private' stamp: 'VU 11/7/2004 14:56'!
isValue: aValue
	"Answer true if the result of performing my getSelector is aValue."
	^ (self value) = aValue
! !

!RadioButtonGroupMorph methodsFor: 'private' stamp: 'nk 7/25/2002 16:01'!
setValue: aValue
	(setSelector isNil or: [ target isNil ]) ifTrue: [ ^self ].

	^(setArgument
		ifNil: [ target perform: setSelector withArguments: { aValue } ]
		ifNotNil: [ target perform: setSelector withArguments: { setArgument. aValue } ]) = aValue
	
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RadioButtonGroupMorph class
	instanceVariableNames: ''!

!RadioButtonGroupMorph class methodsFor: 'examples' stamp: 'nk 7/25/2002 16:23'!
example1
	"RadioButtonGroupMorph example1"
	"Make a RBGM with three values"
	| assoc g |
	assoc _ 'myKey' -> #one.
	g _ RadioButtonGroupMorph new initializeWithValues: { #one. #two. #three }.
	g target: assoc;
		getSelector: #value;
		setSelector: #value:;
		extent: 55@62.
	g openInWorld.
! !
